@@ -37,7 +37,9 @@
 #include <linux/poll.h>
 #include <linux/init.h>
 #include <linux/devfs_fs_kernel.h>
+#include <linux/device.h>
 #include <linux/vmalloc.h>
+#include <linux/fs.h>
 
 #include <linux/comedidev.h>
 
@@ -50,10 +52,6 @@ MODULE_AUTHOR("David Schleef <ds@schleef.org>");
 MODULE_DESCRIPTION("Comedi core module");
 MODULE_LICENSE("GPL");
 
-#ifndef KILL_FASYNC
-#define KILL_FASYNC(a,b,c) kill_fasync(&(a),(b),(c))
-#endif
-
 #ifdef CONFIG_COMEDI_DEBUG
 int comedi_debug;
 MODULE_PARM(comedi_debug, "i");
@@ -170,8 +168,9 @@ static int do_devconfig_ioctl(comedi_device *dev,comedi_devconfig *arg, unsigned
 
 	it.board_name[COMEDI_NAMELEN-1]=0;
 
-	if(it.options[COMEDI_DEVCONF_AUX_DATA] &&
-	   it.options[COMEDI_DEVCONF_AUX_DATA_LENGTH]){
+	if(comedi_aux_data(it.options, 0) &&
+		it.options[COMEDI_DEVCONF_AUX_DATA_LENGTH]){
+		int bit_shift;
 		aux_len = it.options[COMEDI_DEVCONF_AUX_DATA_LENGTH];
 		if(aux_len<0)return -EFAULT;
 
@@ -179,12 +178,17 @@ static int do_devconfig_ioctl(comedi_device *dev,comedi_devconfig *arg, unsigned
 		if(!aux_data)return -EFAULT;
 
 		if(copy_from_user(aux_data,
-		    (void *)it.options[COMEDI_DEVCONF_AUX_DATA], aux_len)){
+			comedi_aux_data(it.options, 0), aux_len)){
 			kfree(aux_data);
 			return -EFAULT;
 		}
-
-		it.options[COMEDI_DEVCONF_AUX_DATA] = (unsigned long)aux_data;
+		it.options[COMEDI_DEVCONF_AUX_DATA_LO] = (unsigned long)aux_data;
+		if(sizeof(void*) > sizeof(int))
+		{
+			bit_shift = sizeof(int) * 8;
+			it.options[COMEDI_DEVCONF_AUX_DATA_HI] = ((unsigned long)aux_data) >> bit_shift;
+		}else
+			it.options[COMEDI_DEVCONF_AUX_DATA_HI] = 0;
 	}
 
 	ret = comedi_device_attach(dev,&it);
@@ -241,8 +245,11 @@ static int do_bufconfig_ioctl(comedi_device *dev,void *arg)
 		if(bc.size > async->max_bufsize)
 			return -EPERM;
 
-		if(s->busy)return -EBUSY;
-
+		if(s->busy)
+		{
+			DPRINTK("subdevice is busy, cannot resize buffer\n");
+			return -EBUSY;
+		}
 		if(async->mmap_count){
 			DPRINTK("subdevice is mmapped, cannot resize buffer\n");
 			return -EBUSY;
@@ -324,16 +331,16 @@ static int do_devinfo_ioctl(comedi_device *dev,comedi_devinfo *arg)
 /*
 	COMEDI_SUBDINFO
 	subdevice info ioctl
-	
+
 	arg:
 		pointer to array of subdevice info structures
-	
+
 	reads:
 		none
-	
+
 	writes:
 		array of subdevice info structures at arg
-		
+
 */
 static int do_subdinfo_ioctl(comedi_device *dev,comedi_subdinfo *arg,void *file)
 {
@@ -367,7 +374,7 @@ static int do_subdinfo_ioctl(comedi_device *dev,comedi_subdinfo *arg,void *file)
 			us->range_type	= 0; /* XXX */
 		}
 		us->flags		= s->flags;
-		
+
 		if(s->busy)
 			us->subd_flags |= SDF_BUSY;
 		if(s->busy == file)
@@ -387,9 +394,9 @@ static int do_subdinfo_ioctl(comedi_device *dev,comedi_subdinfo *arg,void *file)
 
 		us->settling_time_0 = s->settling_time_0;
 	}
-	
+
 	ret=copy_to_user(arg,tmp,dev->n_subdevices*sizeof(comedi_subdinfo));
-	
+
 	kfree(tmp);
 
 	return ret?-EFAULT:0;
@@ -399,10 +406,10 @@ static int do_subdinfo_ioctl(comedi_device *dev,comedi_subdinfo *arg,void *file)
 /*
 	COMEDI_CHANINFO
 	subdevice info ioctl
-	
+
 	arg:
 		pointer to chaninfo structure
-	
+
 	reads:
 		chaninfo structure at arg
 
@@ -434,7 +441,7 @@ static int do_chaninfo_ioctl(comedi_device *dev,comedi_chaninfo *arg)
 		if(copy_to_user(it.flaglist,s->flaglist,s->n_chan*sizeof(unsigned int)))
 			return -EFAULT;
 	}
-			
+
 	if(it.rangelist){
 		int i;
 
@@ -486,10 +493,10 @@ static int do_bufinfo_ioctl(comedi_device *dev,void *arg)
 
 	if(!async){
 		DPRINTK("subdevice does not have async capability\n");
-		bi.buf_int_ptr = 0;
-		bi.buf_user_ptr = 0;
-		bi.buf_int_count = 0;
-		bi.buf_user_count = 0;
+		bi.buf_write_ptr = 0;
+		bi.buf_read_ptr = 0;
+		bi.buf_write_count = 0;
+		bi.buf_read_count = 0;
 		goto copyback;
 	}
 
@@ -509,21 +516,15 @@ static int do_bufinfo_ioctl(comedi_device *dev,void *arg)
 		comedi_buf_write_free(async, bi.bytes_written);
 	}
 
+	bi.buf_write_count = async->buf_write_count;
+	bi.buf_write_ptr = async->buf_write_ptr;
+	bi.buf_read_count = async->buf_read_count;
+	bi.buf_read_ptr = async->buf_read_ptr;
 	if(s==dev->read_subdev){
 		unsigned int n_munge_bytes;
-
-		bi.buf_int_count = async->buf_write_count;
-		bi.buf_int_ptr = async->buf_write_ptr;
-		bi.buf_user_count = async->buf_read_count;
-		bi.buf_user_ptr = async->buf_read_ptr;
-
-		n_munge_bytes = bi.buf_int_count - s->async->munge_count;
+		
+		n_munge_bytes = bi.buf_write_count - s->async->munge_count;
 		comedi_buf_munge(dev, s, n_munge_bytes);
-	}else{
-		bi.buf_int_count = async->buf_read_count;
-		bi.buf_int_ptr = async->buf_read_ptr;
-		bi.buf_user_count = async->buf_write_count;
-		bi.buf_user_ptr = async->buf_write_ptr;
 	}
 
 copyback:
@@ -552,7 +553,6 @@ static int parse_insn(comedi_device *dev,comedi_insn *insn,lsampl_t *data,void *
  */
 /* arbitrary limits */
 #define MAX_SAMPLES 256
-#define MAX_INSNS 10
 static int do_insnlist_ioctl(comedi_device *dev,void *arg,void *file)
 {
 	comedi_insnlist insnlist;
@@ -564,11 +564,6 @@ static int do_insnlist_ioctl(comedi_device *dev,void *arg,void *file)
 	if(copy_from_user(&insnlist,arg,sizeof(comedi_insnlist)))
 		return -EFAULT;
 
-	if(insnlist.n_insns>=MAX_INSNS){
-		DPRINTK("insnlist too long\n");
-		return -EINVAL;
-	}
-
 	data=kmalloc(sizeof(lsampl_t)*MAX_SAMPLES,GFP_KERNEL);
 	if(!data){
 		DPRINTK("kmalloc failed\n");
@@ -618,6 +613,8 @@ static int do_insnlist_ioctl(comedi_device *dev,void *arg,void *file)
 				goto error;
 			}
 		}
+		if(need_resched())
+			schedule();
 	}
 
 error:
@@ -628,6 +625,27 @@ error:
 	return i;
 }
 
+static int check_insn_config_length(comedi_insn *insn, lsampl_t *data)
+{
+	switch(data[0])
+	{
+	case INSN_CONFIG_DIO_OUTPUT:
+	case INSN_CONFIG_DIO_INPUT:
+		if(insn->n == 1) return 0;
+		break;
+	case INSN_CONFIG_DIO_QUERY:
+		if(insn->n == 2) return 0;
+		break;
+	case INSN_CONFIG_BLOCK_SIZE:
+		if( insn->n == 2 ) return 0;
+	//by default we allow the insn since we don't have checks for all possible cases yet 
+	default:
+		return 0;
+		break;
+	}
+	return -EINVAL;
+}
+
 static int parse_insn(comedi_device *dev,comedi_insn *insn,lsampl_t *data,void *file)
 {
 	comedi_subdevice *s;
@@ -724,7 +742,6 @@ static int parse_insn(comedi_device *dev,comedi_insn *insn,lsampl_t *data,void *
 		}
 		/* This looks arbitrary.  It is. */
 		s->busy=&parse_insn;
-
 		switch(insn->insn){
 			case INSN_READ:
 				ret=s->insn_read(dev,s,insn,data);
@@ -737,6 +754,8 @@ static int parse_insn(comedi_device *dev,comedi_insn *insn,lsampl_t *data,void *
 				ret=s->insn_bits(dev,s,insn,data);
 				break;
 			case INSN_CONFIG:
+				ret=check_insn_config_length(insn, data);
+				if(ret) break;
 				ret=s->insn_config(dev,s,insn,data);
 				break;
 			default:
@@ -925,7 +944,7 @@ static int do_cmd_ioctl(comedi_device *dev,void *arg,void *file)
 
 	init_async_buf( async );
 
-	async->cb_mask = COMEDI_CB_EOA|COMEDI_CB_BLOCK|COMEDI_CB_ERROR;
+	async->cb_mask = COMEDI_CB_EOA | COMEDI_CB_BLOCK | COMEDI_CB_ERROR | COMEDI_CB_OVERFLOW;
 	if(async->cmd.flags & TRIG_WAKE_EOS){
 		async->cb_mask |= COMEDI_CB_EOS;
 	}
@@ -1049,13 +1068,13 @@ cleanup:
 /*
 	COMEDI_LOCK
 	lock subdevice
-	
+
 	arg:
 		subdevice number
-	
+
 	reads:
 		none
-	
+
 	writes:
 		none
 
@@ -1071,17 +1090,17 @@ static int do_lock_ioctl(comedi_device *dev,unsigned int arg,void * file)
 		return -EINVAL;
 	s=dev->subdevices+arg;
 
-	if(s->busy)
-		return -EBUSY;
-
 	comedi_spin_lock_irqsave(&big_comedi_lock, flags);
-
-	if(s->lock && s->lock!=file){
-		ret=-EACCES;
+	if(s->busy)
+	{
+		comedi_spin_unlock_irqrestore(&big_comedi_lock, flags);
+		return -EBUSY;
+	}
+	if(s->lock){
+		ret=-EBUSY;
 	}else{
 		s->lock=file;
 	}
-
 	comedi_spin_unlock_irqrestore(&big_comedi_lock, flags);
 
 	if(ret<0)
@@ -1105,7 +1124,7 @@ static int do_lock_ioctl(comedi_device *dev,unsigned int arg,void * file)
 
 	reads:
 		none
-	
+
 	writes:
 		none
 
@@ -1141,13 +1160,13 @@ static int do_unlock_ioctl(comedi_device *dev,unsigned int arg,void * file)
 /*
 	COMEDI_CANCEL
 	cancel acquisition ioctl
-	
+
 	arg:
 		subdevice number
-	
+
 	reads:
 		nothing
-	
+
 	writes:
 		nothing
 
@@ -1175,10 +1194,10 @@ static int do_cancel_ioctl(comedi_device *dev,unsigned int arg,void *file)
 /*
 	COMEDI_POLL ioctl
 	instructs driver to synchronize buffers
-	
+
 	arg:
 		subdevice number
-	
+
 	reads:
 		nothing
 
@@ -1296,7 +1315,7 @@ static int comedi_mmap_v22(struct file * file, struct vm_area_struct *vma)
 
 	n_pages = size >> PAGE_SHIFT;
 	for(i=0;i<n_pages;i++){
-		if(REMAP_PAGE_RANGE(vma, start, __pa(async->buf_page_list[i]),
+		if(remap_pfn_range(vma, start, __pa(async->buf_page_list[i]) >> PAGE_SHIFT,
 				PAGE_SIZE, PAGE_SHARED)){
 			return -EAGAIN;
 		}
@@ -1508,8 +1527,7 @@ static ssize_t comedi_read_v22(struct file * file,char *buf,size_t nbytes,loff_t
 			schedule();
 			continue;
 		}
-
-		comedi_buf_munge(dev, s, n);
+		comedi_buf_munge(dev, s, async->buf_write_count - async->munge_count);
 		m = copy_to_user(buf, async->prealloc_buf +
 			async->buf_read_ptr, n);
 		if(m){
@@ -1620,11 +1638,11 @@ static int comedi_fop_open(struct inode *inode,struct file *file)
 	 */
 	if(dev->attached)
 		goto ok;
-	if(!capable(CAP_SYS_ADMIN) && dev->in_request_module){
+	if(!capable(CAP_SYS_MODULE) && dev->in_request_module){
 		DPRINTK("in request module\n");
 		return -ENODEV;
 	}
-	if(capable(CAP_SYS_ADMIN) && dev->in_request_module)
+	if(capable(CAP_SYS_MODULE) && dev->in_request_module)
 		goto ok;
 
 	dev->in_request_module=1;
@@ -1636,8 +1654,8 @@ static int comedi_fop_open(struct inode *inode,struct file *file)
 
 	dev->in_request_module=0;
 
-	if(!dev->attached && !capable(CAP_SYS_ADMIN)){
-		DPRINTK("not attached and not root\n");
+	if(!dev->attached && !capable(CAP_SYS_MODULE)){
+		DPRINTK("not attached and not CAP_SYS_MODULE\n");
 		return -ENODEV;
 	}
 ok:
@@ -1766,6 +1784,8 @@ static struct file_operations comedi_fops={
 };
 #endif
 
+static struct class_simple *comedi_class;
+
 static int __init comedi_init(void)
 {
 	int i;
@@ -1775,9 +1795,20 @@ static int __init comedi_init(void)
 		printk("comedi: unable to get major %d\n",COMEDI_MAJOR);
 		return -EIO;
 	}
+	comedi_class = class_simple_create(THIS_MODULE, "comedi");
+	if(IS_ERR(comedi_class))
+	{
+		printk("comedi: failed to create class");
+		devfs_unregister_chrdev(COMEDI_MAJOR,"comedi");
+		return PTR_ERR(comedi_class);
+	}
 	comedi_devices=(comedi_device *)kmalloc(sizeof(comedi_device)*COMEDI_NDEVICES,GFP_KERNEL);
 	if(!comedi_devices)
+	{
+		class_simple_destroy(comedi_class);
+		devfs_unregister_chrdev(COMEDI_MAJOR,"comedi");
 		return -ENOMEM;
+	}
 	memset(comedi_devices,0,sizeof(comedi_device)*COMEDI_NDEVICES);
 	for(i=0;i<COMEDI_NDEVICES;i++){
 		comedi_devices[i].minor=i;
@@ -1792,8 +1823,9 @@ static int __init comedi_init(void)
 		sprintf(name, "comedi%d", i);
 		devfs_register(NULL, name, DEVFS_FL_DEFAULT,
 			COMEDI_MAJOR, i, 0666 | S_IFCHR, &comedi_fops, NULL);
+		class_simple_device_add(comedi_class, MKDEV(COMEDI_MAJOR, i), NULL, "comedi%i", i);
 	}
-	
+
 	comedi_rt_init();
 
 	return 0;
@@ -1808,11 +1840,12 @@ static void __exit comedi_cleanup(void)
 
 	for(i=0;i<COMEDI_NDEVICES;i++){
 		char name[20];
+		class_simple_device_remove(MKDEV(COMEDI_MAJOR, i));
 		sprintf(name, "comedi%d", i);
 		devfs_unregister(devfs_find_handle(NULL, name,
 			COMEDI_MAJOR, i, DEVFS_SPECIAL_CHR, 0));
 	}
-
+	class_simple_destroy(comedi_class);
 	devfs_unregister_chrdev(COMEDI_MAJOR,"comedi");
 
 	comedi_proc_cleanup();
@@ -1844,8 +1877,6 @@ void comedi_event(comedi_device *dev,comedi_subdevice *s, unsigned int mask)
 	mask = s->async->events;
 	s->async->events = 0;
 
-	mask |= COMEDI_CB_BLOCK;
-
 	//DPRINTK("comedi_event %x\n",mask);
 
 	if( (s->subdev_flags & SDF_RUNNING) == 0)
