@@ -184,12 +184,16 @@ static int subdev_8255_insn_config(comedi_device *dev,comedi_subdevice *s,
 	}
 
 	switch(data[0]){
-	case COMEDI_INPUT:
+	case INSN_CONFIG_DIO_INPUT:
 		s->io_bits&=~bits;
 		break;
-	case COMEDI_OUTPUT:
+	case INSN_CONFIG_DIO_OUTPUT:
 		s->io_bits|=bits;
 		break;
+	case INSN_CONFIG_DIO_QUERY:
+		data[1] = (s->io_bits & bits) ? COMEDI_OUTPUT : COMEDI_INPUT;
+		return insn->n;
+		break;
 	default:
 		return -EINVAL;
 	}
@@ -253,7 +257,7 @@ static int subdev_8255_cmdtest(comedi_device *dev, comedi_subdevice *s,
 	/* step 3 */
 
 	if(cmd->start_arg != 0){
-		cmd->start_arg = 1;
+		cmd->start_arg = 0;
 		err++;
 	}
 	if(cmd->scan_begin_arg!=0){
