@@ -8,7 +8,7 @@
 /*
 Driver: adv_pci_dio.o
 Description: Advantech PCI-1730, PCI-1733, PCI-1734, PCI-1750, PCI-1751,
-             Advantech PCI-1752, PCI-1753/E, PCI-1754, PCI-1756, PCI-1762
+             PCI-1752, PCI-1753/E, PCI-1754, PCI-1756, PCI-1762
 Author: Michal Dobes <dobes@tesnet.cz>
 Devices: [Advantech] PCI-1730 (pci1730), PCI-1733 (pci1733),
   PCI-1734 (pci1734), PCI-1750 (pci1750), PCI-1751 (pci1751),
@@ -93,15 +93,15 @@ typedef enum {
 #define PCI1751_ICR	  32		/* W:   Interrupt control register */
 #define PCI1751_ISR	  32		/* R:   Interrupt status register */
 #define PCI1753_DIO	   0		/* R/W: begin of 8255 registers block */
-#define PCI1753_ICR0	  17		/* R/W: Interrupt control register group 0 */
-#define PCI1753_ICR1	  18		/* R/W: Interrupt control register group 0 */
-#define PCI1753_ICR2	  19		/* R/W: Interrupt control register group 0 */
-#define PCI1753_ICR3	  48		/* R/W: Interrupt control register group 0 */
+#define PCI1753_ICR0	  16		/* R/W: Interrupt control register group 0 */
+#define PCI1753_ICR1	  17		/* R/W: Interrupt control register group 1 */
+#define PCI1753_ICR2	  18		/* R/W: Interrupt control register group 2 */
+#define PCI1753_ICR3	  19		/* R/W: Interrupt control register group 3 */
 #define PCI1753E_DIO	  32		/* R/W: begin of 8255 registers block */
-#define PCI1753E_ICR0	  49		/* R/W: Interrupt control register group 0 */
-#define PCI1753E_ICR1	  50		/* R/W: Interrupt control register group 0 */
-#define PCI1753E_ICR2	  51		/* R/W: Interrupt control register group 0 */
-#define PCI1753E_ICR3	  52		/* R/W: Interrupt control register group 0 */
+#define PCI1753E_ICR0	  48		/* R/W: Interrupt control register group 0 */
+#define PCI1753E_ICR1	  49		/* R/W: Interrupt control register group 1 */
+#define PCI1753E_ICR2	  50		/* R/W: Interrupt control register group 2 */
+#define PCI1753E_ICR3	  51		/* R/W: Interrupt control register group 3 */
 
 
 // Advantech PCI-1752/4/6
@@ -896,7 +896,8 @@ static int pci_dio_attach(comedi_device *dev, comedi_devconfig *it)
 		return -ENOMEM;
 	}
 
-	pci_for_each_dev(pcidev) {
+	for(pcidev = pci_get_device(PCI_ANY_ID, PCI_ANY_ID, NULL); pcidev != NULL ; 
+		pcidev = pci_get_device(PCI_ANY_ID, PCI_ANY_ID, pcidev)) {
 		if ((pcidev->vendor!=this_board->vendor_id)||
 		    (pcidev->device!=this_board->device_id))
 			continue;
@@ -910,6 +911,7 @@ static int pci_dio_attach(comedi_device *dev, comedi_devconfig *it)
 		if (ret==1) { found=1; break; }
 		if (ret>1) { 
 			pci_dio_detach(dev);
+			pci_dev_put(pcidev);
 			return -EIO;
 		}
 	}
@@ -927,6 +929,7 @@ static int pci_dio_attach(comedi_device *dev, comedi_devconfig *it)
 	
 	if (pci_request_regions(pcidev, driver_pci_dio.driver_name)) {
 		pci_dio_detach(dev);
+		pci_dev_put(pcidev);
 		rt_printk(", Error: Cann't allocate PCI device!\n");
 		return -EIO;
 	}
@@ -1047,8 +1050,10 @@ static int pci_dio_detach(comedi_device *dev)
 		if (devpriv->enabled)
 			pci_disable_device(devpriv->pcidev);
 
-		if (devpriv->pcidev)
+		if (devpriv->pcidev) {
 			pci_release_regions(devpriv->pcidev);
+			pci_dev_put(devpriv->pcidev);
+		}
 		
 		if (devpriv->prev) { devpriv->prev->next=devpriv->next; }
 				    { pci_priv=devpriv->next; }
