@@ -28,8 +28,7 @@
 */
 /*
 Driver: cb_pcidas.o
-Description: Driver for the ComputerBoards/MeasurementComputing cards
-  of the PCI-DAS series with the AMCC S5933 PCI controller.
+Description: MeasurementComputing PCI-DAS series with the AMCC S5933 PCI controller
 Author: Ivan Martinez <imr@oersted.dtu.dk>,
   Frank Mori Hess <fmhess@users.sourceforge.net>
 Updated: 2003-3-11
@@ -526,7 +525,8 @@ static int cb_pcidas_attach(comedi_device *dev, comedi_devconfig *it)
  */
 	printk("\n");
 
-	pci_for_each_dev(pcidev)
+	for(pcidev = pci_find_device(PCI_ANY_ID, PCI_ANY_ID, NULL); pcidev != NULL ; 
+		pcidev = pci_find_device(PCI_ANY_ID, PCI_ANY_ID, pcidev)) 
 	{
 		// is it not a computer boards card?
 		if(pcidev->vendor != PCI_VENDOR_ID_CB)
@@ -768,6 +768,8 @@ static int cb_pcidas_detach(comedi_device *dev)
 		comedi_free_irq(dev->irq, dev);
 	if(dev->subdevices)
 		subdev_8255_cleanup(dev,dev->subdevices + 2);
+	if(devpriv && devpriv->pci_dev)
+		pci_dev_put(devpriv->pci_dev);
 
 	return 0;
 }
@@ -782,17 +784,21 @@ static int cb_pcidas_ai_rinsn(comedi_device *dev, comedi_subdevice *s,
 	int n,i;
 	unsigned int bits;
 	static const int timeout = 10000;
-
+	int channel;
 	// enable calibration input if appropriate
 	if( insn->chanspec & CR_ALT_SOURCE )
+	{
 		outw( cal_enable_bits( dev ),
 			devpriv->control_status + CALIBRATION_REG);
-	else
+		channel = 0;
+	}else
+	{
 		outw( 0, devpriv->control_status + CALIBRATION_REG);
-
+		channel = CR_CHAN(insn->chanspec);
+	}
 	// set mux limits and gain
-	bits = BEGIN_SCAN(CR_CHAN(insn->chanspec)) |
-		END_SCAN(CR_CHAN(insn->chanspec)) |
+	bits = BEGIN_SCAN(channel) |
+		END_SCAN(channel) |
 		GAIN_BITS(CR_RANGE(insn->chanspec));
 	// set unipolar/bipolar
 	if(CR_RANGE(insn->chanspec) & IS_UNIPOLAR)
@@ -872,8 +878,8 @@ static int cb_pcidas_ao_nofifo_winsn(comedi_device *dev, comedi_subdevice *s,
 	// set channel and range
 	channel = CR_CHAN(insn->chanspec);
 	comedi_spin_lock_irqsave( &dev->spinlock, flags );
-	devpriv->ao_control_bits &= ~DAC_MODE_UPDATE_BOTH & ~DAC_RANGE_MASK( channel );
-	devpriv->ao_control_bits |= DACEN | DAC_RANGE( channel, CR_RANGE( insn->chanspec ) );
+	devpriv->ao_control_bits &= ~DAC_MODE_UPDATE_BOTH & ~DAC_RANGE_MASK(channel);
+	devpriv->ao_control_bits |= DACEN | DAC_RANGE(channel, CR_RANGE(insn->chanspec));
 	outw( devpriv->ao_control_bits, devpriv->control_status + DAC_CSR );
 	comedi_spin_unlock_irqrestore( &dev->spinlock, flags );
 
@@ -889,18 +895,21 @@ static int cb_pcidas_ao_nofifo_winsn(comedi_device *dev, comedi_subdevice *s,
 static int cb_pcidas_ao_fifo_winsn(comedi_device *dev, comedi_subdevice *s,
 	comedi_insn *insn, lsampl_t *data)
 {
-	int bits, channel;
+	int channel;
+	unsigned long flags;
 
 	// clear dac fifo
 	outw(0, devpriv->ao_registers + DACFIFOCLR);
 
 	// set channel and range
 	channel = CR_CHAN(insn->chanspec);
-	bits = DACEN;
-	bits |= DAC_RANGE(channel, CR_RANGE(insn->chanspec));
-	bits |= DAC_CHAN_EN(channel);
-	bits |= DAC_START;	// not sure if this is necessary
-	outw(bits, devpriv->control_status + DAC_CSR);
+	comedi_spin_lock_irqsave(&dev->spinlock, flags);
+	devpriv->ao_control_bits &= ~DAC_CHAN_EN(0) & ~DAC_CHAN_EN(1) & ~DAC_RANGE_MASK(channel) &
+		~DAC_PACER_MASK;
+	devpriv->ao_control_bits |= DACEN | DAC_RANGE(channel, CR_RANGE(insn->chanspec)) |
+		DAC_CHAN_EN(channel) | DAC_START;
+	outw(devpriv->ao_control_bits, devpriv->control_status + DAC_CSR);
+	comedi_spin_unlock_irqrestore( &dev->spinlock, flags );
 
 	// remember value for readback
 	devpriv->ao_value[channel] = data[0];
