@@ -26,7 +26,7 @@
 */
 /*
 Driver: cb_pcidda.o
-Description: ComputerBoards/MeasurementComputing PCI-DDA series
+Description: MeasurementComputing PCI-DDA series
 Author: Ivan Martinez <ivanmr@altavista.com>, Frank Mori Hess <fmhess@users.sourceforge.net>
 Status: Supports 08/16, 04/16, 02/16, 08/12, 04/12, and 02/12
 Devices: [Measurement Computing] PCI-DDA08/12 (cb_pcidda), PCI-DDA04/12,
@@ -284,7 +284,8 @@ static int cb_pcidda_attach(comedi_device *dev, comedi_devconfig *it)
  */
 	printk("\n");
 
-	pci_for_each_dev(pcidev){
+	for(pcidev = pci_get_device(PCI_ANY_ID, PCI_ANY_ID, NULL); pcidev != NULL ; 
+		pcidev = pci_get_device(PCI_ANY_ID, PCI_ANY_ID, pcidev)) {
 		if(pcidev->vendor==PCI_VENDOR_ID_CB){
 			if(it->options[0] || it->options[1]){
 				if(pcidev->bus->number==it->options[0] &&
@@ -309,6 +310,7 @@ static int cb_pcidda_attach(comedi_device *dev, comedi_devconfig *it)
 	}
 	printk("Not a supported ComputerBoards/MeasurementComputing card on "
 		"requested position\n");
+	pci_dev_put(pcidev);
 	return -EIO;
 
 found:
@@ -407,6 +409,8 @@ static int cb_pcidda_detach(comedi_device *dev)
 			release_region(devpriv->digitalio, DIGITALIO_SIZE);
 		if(devpriv->dac)
 			release_region(devpriv->dac, 8 + thisboard->ao_chans*2);
+		if(devpriv->pci_dev)
+			pci_dev_put(devpriv->pci_dev);
 	}
 	// cleanup 8255
 	if(dev->subdevices)
