@@ -30,10 +30,10 @@ Devices: [IOTech] DAQBoard/2000 (daqboard2000)
 Much of the functionality of this driver was determined from reading
 the source code for the Windows driver.
 
-The FPGA on the board requires initialization code, which can either
-be compiled into the driver or loaded by comedi_config using the -i
-option.  The latter is recommended, in order to save a bit of kernel
-memory.
+The FPGA on the board requires initialization code, which can 
+be loaded by comedi_config using the -i
+option.  The initialization code is available from http://www.comedi.org
+in the comedi_nonfree_firmware tarball.
 
 Configuration options:
     none
@@ -78,7 +78,7 @@ Configuration options:
 		  |		 | +-------- Simultaneous sample and hold
 		  |		 +---------- Signed data format
 		  +------------------------- Correction offset low
-			       
+
         Word2:
           +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
           ! | | | ! | | | ! | | | ! | | | !
@@ -323,6 +323,7 @@ typedef struct {
   enum {
     card_daqboard_2000
   } card;
+  struct pci_dev *pci_dev;
   void *daq;
   void *plx;
   lsampl_t ao_readback[2];
@@ -438,7 +439,7 @@ static int daqboard2000_ao_insn_write(comedi_device *dev, comedi_subdevice *s,
   int timeout;
 
   for(i=0;i<insn->n;i++){
-    /*  
+    /*
      * OK, since it works OK without enabling the DAC's, let's keep
      * it as simple as possible...
      */
@@ -546,7 +547,7 @@ static int initialize_daqboard2000(comedi_device *dev,
     daqboard2000_pulseProgPin(dev);
     if(daqboard2000_pollCPLD(dev, DAQBOARD2000_CPLD_INIT)) {
       for (i = 0 ; i < len ; i++) {
-        if (cpld_array[i] == 0xff && cpld_array[i+1] == 0x20) { 
+        if (cpld_array[i] == 0xff && cpld_array[i+1] == 0x20) {
 #ifdef DEBUG_EEPROM
           printk("Preamble found at %d\n", i);
 #endif
@@ -685,7 +686,7 @@ static int daqboard2000_attach(comedi_device *dev, comedi_devconfig *it)
 
   /* FIXME: we should handle multiple cards, have to make David decide 
             how, so we will be consistent among all PCI card drivers... */
-  card = pci_find_device(0x1616, 0x0409, NULL);
+  card = pci_get_device(0x1616, 0x0409, NULL);
 
   if (!card) {
     printk(" no daqboard2000 found\n");
@@ -694,6 +695,7 @@ static int daqboard2000_attach(comedi_device *dev, comedi_devconfig *it)
   }
 
   if((result = pci_enable_device(card))<0){
+    pci_dev_put(card);
     goto out;
   }
 
@@ -714,11 +716,17 @@ static int daqboard2000_attach(comedi_device *dev, comedi_devconfig *it)
   }else{
     printk(" abnormal pci header type !?!?\n");
     result=-EIO;
+    pci_dev_put(card);
     goto out;
   }
   
   result = alloc_private(dev,sizeof(daqboard2000_private));
-  if(result<0)goto out;
+  if(result<0){
+    pci_dev_put(card);
+    goto out;
+  }
+
+  devpriv->pci_dev = card;
 
   result = alloc_subdevices(dev, 3);
   if(result<0)goto out;
@@ -734,7 +742,7 @@ static int daqboard2000_attach(comedi_device *dev, comedi_devconfig *it)
     printk("Interrupt before is: %x\n", interrupt);
   */
 
-  aux_data = (void *)it->options[COMEDI_DEVCONF_AUX_DATA];
+  aux_data = comedi_aux_data(it->options, 0);
   aux_len = it->options[COMEDI_DEVCONF_AUX_DATA_LENGTH];
 
   if(aux_data && aux_len){
@@ -752,8 +760,8 @@ static int daqboard2000_attach(comedi_device *dev, comedi_devconfig *it)
     printk("Interrupt after is: %x\n", interrupt);
   */
 
-  dev->iobase = (int)devpriv->daq;
-    
+  dev->iobase = (unsigned long)devpriv->daq;
+
   dev->board_name = this_board->name;
 
   s = dev->subdevices + 0;
@@ -800,6 +808,9 @@ static int daqboard2000_detach(comedi_device * dev)
   if (dev->irq) {
     free_irq(dev->irq, dev);
   }
+  if (devpriv && devpriv->pci_dev) {
+    pci_dev_put(devpriv->pci_dev);
+  }
   return 0;
 }
 
