@@ -39,13 +39,13 @@ Devices: [Keithley Metrabyte] DAS-16 (das-16), DAS-16G (das-16g),
   CIO-DAS1601/12 (cio-das1601/12), CIO-DAS1602/12 (cio-das1602/12),
   CIO-DAS1602/16 (cio-das1602/16), CIO-DAS16/330 (cio-das16/330)
 Status: works
-Updated: 2002-09-30
+Updated: 2003-10-12
 
 A rewrite of the das16 and das1600 drivers.
 Options:
 	[0] - base io address
-	[1] - irq (optional)
-	[2] - dma (optional)
+	[1] - irq (does nothing, irq is not used anymore)
+	[2] - dma (optional, required for comedi_command support)
 	[3] - master clock speed in MHz (optional, 1 or 10, ignored if
 		board can probe clock, defaults to 1)
 	[4] - analog input range lowest voltage in microvolts (optional,
@@ -56,18 +56,13 @@ Options:
 		gain)
 	[6] - analog output range lowest voltage in microvolts (optional)
 	[7] - analog output range highest voltage in microvolts (optional)
-	[8] - use timer mode for DMA, needed e.g. for buggy DMA controller
-		in NS CS5530A (Geode Companion), and for 'jr' cards that
-		lack a hardware fifo.  If set, also allows
-		comedi_command() to be run without an irq.
+	[8] - use timer mode for DMA.  Timer mode is needed e.g. for
+		buggy DMA controllers in NS CS5530A (Geode Companion), and for
+		'jr' cards that lack a hardware fifo.  This option is no
+		longer needed, since timer mode is _always_ used.
 
 Passing a zero for an option is the same as leaving it unspecified.
 
-Both a dma channel and an irq (or use of 'timer mode', option 8) are required
-for timed or externally triggered conversions.  The JR versions of the boards
-should use 'timer mode' instead of an irq, due to their lack of a hardware
-fifo.
-
 */
 /*
 
@@ -334,6 +329,8 @@ static int das16_ai_rinsn(comedi_device *dev,comedi_subdevice *s,comedi_insn *in
 static int das16_cmd_test(comedi_device *dev,comedi_subdevice *s,comedi_cmd *cmd);
 static int das16_cmd_exec(comedi_device *dev,comedi_subdevice *s);
 static int das16_cancel(comedi_device *dev, comedi_subdevice *s);
+static void das16_ai_munge(comedi_device *dev, comedi_subdevice *s, void *array,
+	unsigned int num_bytes, unsigned int start_chan_index);
 
 static void das16_reset(comedi_device *dev);
 static irqreturn_t das16_dma_interrupt(int irq, void *d, struct pt_regs *regs);
@@ -344,11 +341,6 @@ static unsigned int das16_set_pacer(comedi_device *dev, unsigned int ns, int fla
 static int das1600_mode_detect(comedi_device *dev);
 static unsigned int das16_suggest_transfer_size(comedi_device *dev, comedi_cmd cmd);
 
-static void init_munge_info( struct munge_info *info );
-static void write_byte_to_buffer( comedi_device *dev, comedi_subdevice *subd,
-	uint8_t raw_byte );
-static void das16_write_array_to_buffer( comedi_device *dev, void *data, unsigned int num_bytes );
-
 static void reg_dump(comedi_device *dev);
 
 typedef struct das16_board_struct{
@@ -716,15 +708,19 @@ static comedi_driver driver_das16={
 
 #define DAS16_TIMEOUT 1000
 
-static const int timer_period = HZ / 20 + 1;	// period for timer interrupt in jiffies (about 1/20 of a second)
-
+/* Period for timer interrupt in jiffies.  It's a function
+ * to deal with possibility of dynamic HZ patches  */
+static inline int timer_period(void)
+{
+	return HZ / 20;
+}
 struct das16_private_struct
 {
 	unsigned int	ai_unipolar;	// unipolar flag
 	unsigned int	ai_singleended;	// single ended flag
 	unsigned int	clockbase;	// master clock speed in ns
 	volatile unsigned int	control_state;	// dma, interrupt and trigger control bits
-	volatile unsigned long	adc_byte_count;	// number of samples remaining
+	volatile unsigned long	adc_byte_count;	// number of bytes remaining
 	unsigned int divisor1;	// divisor dividing master clock to get conversion frequency
 	unsigned int divisor2;	// divisor dividing master clock to get conversion frequency
 	unsigned int dma_chan;	// dma channel
@@ -735,11 +731,10 @@ struct das16_private_struct
 	// user-defined analog input and output ranges defined from config options
 	comedi_lrange *user_ai_range_table;
 	comedi_lrange *user_ao_range_table;
-	struct munge_info ai_munge_info;
 
 	struct timer_list timer;                // for timed interrupt
-	volatile unsigned int timer_running : 1;
-	volatile unsigned int timer_mode : 1;   // true if using timer mode
+	volatile short timer_running;
+	volatile short timer_mode;   // true if using timer mode
 };
 #define devpriv ((struct das16_private_struct *)(dev->private))
 #define thisboard ((struct das16_board_struct *)(dev->board_ptr))
@@ -962,7 +957,6 @@ static int das16_cmd_exec(comedi_device *dev,comedi_subdevice *s)
 	/* clear flip-flop to make sure 2-byte registers for
 	 * count and address get set correctly */
 	clear_dma_ff(devpriv->dma_chan);
-	init_munge_info( &devpriv->ai_munge_info );
 	devpriv->current_buffer = 0;
 	set_dma_addr( devpriv->dma_chan,
 		devpriv->dma_buffer_addr[ devpriv->current_buffer ] );
@@ -976,7 +970,7 @@ static int das16_cmd_exec(comedi_device *dev,comedi_subdevice *s)
 	if( devpriv->timer_mode )
 	{
 		devpriv->timer_running = 1;
-		devpriv->timer.expires = jiffies + timer_period;
+		devpriv->timer.expires = jiffies + timer_period();
 		add_timer(&devpriv->timer);
 		devpriv->control_state &= ~DAS16_INTE;
 	}else
@@ -1163,7 +1157,6 @@ static irqreturn_t das16_dma_interrupt(int irq, void *d, struct pt_regs *regs)
 
 	/* clear interrupt */
 	outb(0x00, dev->iobase + DAS16_STATUS);
-
 	das16_interrupt(dev);
 	return IRQ_HANDLED;
 }
@@ -1175,7 +1168,7 @@ static void das16_timer_interrupt(unsigned long arg)
 	das16_interrupt(dev);
 
 	if(devpriv->timer_running)
-		mod_timer(&devpriv->timer, jiffies + timer_period);
+		mod_timer(&devpriv->timer, jiffies + timer_period());
 }
 
 static void das16_interrupt( comedi_device *dev )
@@ -1231,7 +1224,7 @@ static void das16_interrupt( comedi_device *dev )
 		num_bytes = devpriv->dma_transfer_size - residue;
 
 	if(cmd->stop_src == TRIG_COUNT &&
-		num_bytes > devpriv->adc_byte_count)
+		num_bytes >= devpriv->adc_byte_count)
 	{
 		num_bytes = devpriv->adc_byte_count;
 		async->events |= COMEDI_CB_EOA;
@@ -1253,12 +1246,17 @@ static void das16_interrupt( comedi_device *dev )
 			devpriv->dma_buffer_addr[ devpriv->current_buffer ] );
 		set_dma_count( devpriv->dma_chan, devpriv->dma_transfer_size );
 		enable_dma(devpriv->dma_chan);
+		/* reenable conversions for das1600 mode, (stupid hardware) */
+		if(thisboard->size > 0x400 && devpriv->timer_mode == 0)
+		{
+			outb(0x00, dev->iobase + DAS1600_CONV);
+		}
 	}
 	release_dma_lock(dma_flags);
 
 	comedi_spin_unlock_irqrestore( &dev->spinlock, spin_flags );
 
-	das16_write_array_to_buffer( dev,
+	cfc_write_array_to_buffer( s,
 		devpriv->dma_buffer[ buffer_index ], num_bytes );
 
 	cfc_handle_events( dev, s );
@@ -1366,10 +1364,16 @@ static int das16_attach(comedi_device *dev, comedi_devconfig *it)
 	comedi_krange *user_ai_range, *user_ao_range;
 
 	iobase = it->options[0];
+#if 0
 	irq = it->options[1];
 	timer_mode = it->options[8];
+#endif
+	/* always use time_mode since using irq can drop samples while
+	 * waiting for dma done interrupt (due to hardware limitations) */
+	irq = 0;
+	timer_mode = 1;
 	if( timer_mode ) irq = 0;
-	
+
 	printk("comedi%d: das16:",dev->minor);
 
 	// check that clock setting is valid
@@ -1519,8 +1523,8 @@ static int das16_attach(comedi_device *dev, comedi_devconfig *it)
 		init_timer(&(devpriv->timer));
 		devpriv->timer.function = das16_timer_interrupt;
 		devpriv->timer.data = (unsigned long) dev;
-		devpriv->timer_mode = timer_mode ? 1 : 0;
 	}
+	devpriv->timer_mode = timer_mode ? 1 : 0;
 
 	if((ret=alloc_subdevices(dev, 5))<0)
 		return ret;
@@ -1553,6 +1557,7 @@ static int das16_attach(comedi_device *dev, comedi_devconfig *it)
 		s->do_cmdtest = das16_cmd_test;
 		s->do_cmd = das16_cmd_exec;
 		s->cancel = das16_cancel;
+		s->munge = das16_ai_munge;
 	}else{
 		s->type=COMEDI_SUBD_UNUSED;
 	}
@@ -1712,48 +1717,24 @@ static unsigned int das16_suggest_transfer_size(comedi_device *dev, comedi_cmd c
 
 	if( cmd.stop_src == TRIG_COUNT && size > devpriv->adc_byte_count )
 		size = devpriv->adc_byte_count;
-	
+
 	return size;
 }
 
-static void init_munge_info( struct munge_info *info )
+static void das16_ai_munge(comedi_device *dev, comedi_subdevice *s, void *array,
+	unsigned int num_bytes, unsigned int start_chan_index)
 {
-	info->have_byte = 0;
-}
+	unsigned int i, num_samples = num_bytes / sizeof(sampl_t);
+	sampl_t *data = array;
 
-/* we want to be able to write one byte at a time to buffer to deal with
- * possibility that 8-bit dma transfer will be interrupted inbetween
- * least significant and most significant byte of a sample */
-static void write_byte_to_buffer( comedi_device *dev, comedi_subdevice *subd, uint8_t raw_byte )
-{
-	sampl_t data;
-	struct munge_info *info = &devpriv->ai_munge_info;
-
-	if( info->have_byte == 0 )
+	for(i = 0; i < num_samples; i++)
 	{
-		info->byte = raw_byte;
-		info->have_byte = 1;
-	}else
-	{
-		info->have_byte = 0;
+		data[i] = le16_to_cpu(data[i]);
 		if( thisboard->ai_nbits == 12 )
 		{
-			data = ( raw_byte << 4 ) & 0xff0;
-			data |= ( info->byte >> 4 ) & 0xf;
-		}else
-		{
-			data = ( raw_byte << 8 ) & 0xff00;
-			data |= info->byte & 0xff;
+			data[i] = (data[i] >> 4) & 0xfff;
 		}
-		cfc_write_to_buffer( subd, data );
 	}
 }
 
-static void das16_write_array_to_buffer( comedi_device *dev, void *data, unsigned int num_bytes )
-{
-	unsigned int i;
-	uint8_t *array = data;
 
-	for( i = 0; i < num_bytes; i++ )
-		write_byte_to_buffer( dev, dev->read_subdev, array[i] );
-};
