@@ -24,7 +24,10 @@
 Driver: dt282x.o
 Description: Data Translation DT2821 series (including DT-EZ)
 Author: ds
-Devices: [Data Translation] DT2821 (dt2821), DT2823 (dt2823),
+Devices: [Data Translation] DT2821 (dt2821),
+  DT2821-F-16SE (dt2821-f), DT2821-F-8DI (dt2821-f),
+  DT2821-G-16SE (dt2821-f), DT2821-G-8DI (dt2821-g),
+  DT2823 (dt2823),
   DT2824-PGH (dt2824-pgh), DT2824-PGL (dt2824-pgl), DT2825 (dt2825),
   DT2827 (dt2827), DT2828 (dt2828), DT21-EZ (dt21-ez), DT23-EZ (dt23-ez),
   DT24-EZ (dt24-ez), DT24-EZ-PGL (dt24-ez-pgl)
@@ -58,6 +61,7 @@ Notes:
 #include <linux/ioport.h>
 #include <linux/interrupt.h>
 #include <asm/dma.h>
+#include "comedi_fc.h"
 
 #define DEBUG
 
@@ -202,6 +206,24 @@ typedef struct {
 static boardtype_t boardtypes[] =
 {
 	{	name:		"dt2821",
+		adbits:		12,
+		adchan_se:	16,
+		adchan_di:	8,
+		ai_speed:	20000,
+		ispgl:		0,
+		dachan:		2,
+		dabits:		12,
+	},
+	{	name:		"dt2821-f",
+		adbits:		12,
+		adchan_se:	16,
+		adchan_di:	8,
+		ai_speed:	6500,
+		ispgl:		0,
+		dachan:		2,
+		dabits:		12,
+	},
+	{	name:		"dt2821-g",
 		adbits:		12,
 		adchan_se:	16,
 		adchan_di:	8,
@@ -214,7 +236,7 @@ static boardtype_t boardtypes[] =
 		adbits:		16,
 		adchan_se:	0,
 		adchan_di:	4,
-		ai_speed:	4000,
+		ai_speed:	10000,
 		ispgl:		0,
 		dachan:		2,
 		dabits:		16,
@@ -223,7 +245,7 @@ static boardtype_t boardtypes[] =
 		adbits:		12,
 		adchan_se:	16,
 		adchan_di:	8,
-		ai_speed:	4000,
+		ai_speed:	20000,
 		ispgl:		0,
 		dachan:		0,
 		dabits:		0,
@@ -232,7 +254,7 @@ static boardtype_t boardtypes[] =
 		adbits:		12,
 		adchan_se:	16,
 		adchan_di:	8,
-		ai_speed:	4000,
+		ai_speed:	20000,
 		ispgl:		1,
 		dachan:		0,
 		dabits:		0,
@@ -241,8 +263,8 @@ static boardtype_t boardtypes[] =
 		adbits:		12,
 		adchan_se:	16,
 		adchan_di:	8,
-		ai_speed:	4000,
-		ispgl:		0,
+		ai_speed:	20000,
+		ispgl:		1,
 		dachan:		2,
 		dabits:		12,
 	},
@@ -250,7 +272,7 @@ static boardtype_t boardtypes[] =
 		adbits:		16,
 		adchan_se:	0,
 		adchan_di:	4,
-		ai_speed:	4000,
+		ai_speed:	10000,
 		ispgl:		0,
 		dachan:		2,
 		dabits:		12,
@@ -259,16 +281,25 @@ static boardtype_t boardtypes[] =
 		adbits:		12,
 		adchan_se:	4,
 		adchan_di:	0,
-		ai_speed:	4000,
+		ai_speed:	10000,
 		ispgl:		0,
 		dachan:		2,
 		dabits:		12,
 	},
+	{	name:		"dt2829",
+		adbits:		16,
+		adchan_se:	8,
+		adchan_di:	0,
+		ai_speed:	33250,
+		ispgl:		0,
+		dachan:		2,
+		dabits:		16,
+	},
 	{	name:		"dt21-ez",
 		adbits:		12,
 		adchan_se:	16,
 		adchan_di:	8,
-		ai_speed:	4000,
+		ai_speed:	10000,
 		ispgl:		0,
 		dachan:		2,
 		dabits:		12,
@@ -277,7 +308,7 @@ static boardtype_t boardtypes[] =
 		adbits:		16,
 		adchan_se:	16,
 		adchan_di:	8,
-		ai_speed:	4000,
+		ai_speed:	10000,
 		ispgl:		0,
 		dachan:		0,
 		dabits:		0,
@@ -286,7 +317,7 @@ static boardtype_t boardtypes[] =
 		adbits:		12,
 		adchan_se:	16,
 		adchan_di:	8,
-		ai_speed:	4000,
+		ai_speed:	10000,
 		ispgl:		0,
 		dachan:		0,
 		dabits:		0,
@@ -295,7 +326,7 @@ static boardtype_t boardtypes[] =
 		adbits:		12,
 		adchan_se:	16,
 		adchan_di:	8,
-		ai_speed:	4000,
+		ai_speed:	10000,
 		ispgl:		1,
 		dachan:		0,
 		dabits:		0,
@@ -314,21 +345,21 @@ typedef struct {
 
 	sampl_t ao[2];
 
-	int dacsr;		/* software copies of registers */
-	int adcsr;
-	int supcsr;
+	volatile int dacsr;		/* software copies of registers */
+	volatile int adcsr;
+	volatile int supcsr;
 
-	int ntrig;
-	int nread;
+	volatile int ntrig;
+	volatile int nread;
 
 	struct{
 		int chan;
 		short *buf;	/* DMA buffer */
-		int size;	/* size of current transfer */
+		volatile int size;	/* size of current transfer */
 	}dma[2];
 	int dma_maxsize;	/* max size of DMA transfer (in bytes) */
 	int usedma;		/* driver uses DMA              */
-	int current_dma_chan;
+	volatile int current_dma_index;
 	int dma_dir;
 } dt282x_private;
 
@@ -378,18 +409,17 @@ static int prep_ao_dma(comedi_device * dev,int chan,int size);
 static int dt282x_ai_cancel(comedi_device * dev, comedi_subdevice * s);
 static int dt282x_ao_cancel(comedi_device * dev, comedi_subdevice * s);
 static int dt282x_ns_to_timer(int *nanosec,int round_mode);
+static void dt282x_disable_dma(comedi_device *dev);
 
 
 static int dt282x_grab_dma(comedi_device *dev,int dma1,int dma2);
 
-static void dt282x_copy_to_buffer(comedi_device *dev,sampl_t *buf,
+static void dt282x_munge(comedi_device *dev,sampl_t *buf,
 	unsigned int nbytes)
 {
-	comedi_async *async = dev->subdevices[0].async;
 	unsigned int i;
 	unsigned short mask=(1<<boardtype.adbits)-1;
 	unsigned short sign=1<<(boardtype.adbits-1);
-	unsigned short *abuf;
 	int n;
 
 	if(devpriv->ad_2scomp){
@@ -398,19 +428,11 @@ static void dt282x_copy_to_buffer(comedi_device *dev,sampl_t *buf,
 		sign = 0;
 	}
 
-	abuf = async->prealloc_buf + async->buf_write_ptr;
-	if(async->buf_write_ptr + nbytes >= async->prealloc_bufsz){
-		n = (async->prealloc_bufsz - async->buf_write_ptr - nbytes)/2;
-		for(i=0;i<n;i++){
-			abuf[i] = (buf[i]&mask)^sign;
-		}
-		nbytes -= n;
-		abuf = async->prealloc_buf;
-		buf = buf + i;
-	}
+	if( nbytes % 2 )
+		comedi_error( dev, "bug! odd number of bytes from dma xfer" );
 	n = nbytes/2;
 	for(i=0;i<n;i++){
-		abuf[i] = (buf[i]&mask)^sign;
+		buf[i] = (buf[i]&mask)^sign;
 	}
 }
 
@@ -428,28 +450,24 @@ static void dt282x_ao_dma_interrupt(comedi_device * dev)
 		return;
 	}
 
-	i=devpriv->current_dma_chan;
+	i=devpriv->current_dma_index;
 	ptr=devpriv->dma[i].buf;
 
 	disable_dma(devpriv->dma[i].chan);
 
-	devpriv->current_dma_chan=1-i;
+	devpriv->current_dma_index=1-i;
 
-	size = comedi_buf_read_n_available(s->async);
+	size = comedi_buf_read_n_available(s);
 	if(size>devpriv->dma_maxsize)size=devpriv->dma_maxsize;
 	if( size == 0){
 		rt_printk("dt282x: AO underrun\n");
 		dt282x_ao_cancel(dev,s);
 		s->async->events |= COMEDI_CB_OVERFLOW;
-		comedi_event(dev,s,s->async->events);
 		return;
 	}
 	comedi_buf_memcpy_from(s->async, 0, ptr, size);
 	comedi_buf_read_free(s->async, size);
 	prep_ao_dma(dev,i,size);
-	enable_dma(devpriv->dma[i].chan);
-
-	comedi_event(dev,s,s->async->events);
 	return;
 }
 
@@ -468,24 +486,21 @@ static void dt282x_ai_dma_interrupt(comedi_device * dev)
 		return;
 	}
 
-	i = devpriv->current_dma_chan;
+	i = devpriv->current_dma_index;
 	ptr = devpriv->dma[i].buf;
 	size = devpriv->dma[i].size;
 
 	disable_dma(devpriv->dma[i].chan);
 
-	devpriv->current_dma_chan = 1-i;
+	devpriv->current_dma_index = 1-i;
 
-	ret = comedi_buf_write_alloc(s->async, size);
-	if(!ret){
-		rt_printk("dt282x: AI buffer overflow\n");
-		s->async->events |= COMEDI_CB_OVERFLOW;
-		comedi_event(dev,s,s->async->events);
+	dt282x_munge(dev, ptr, size );
+	ret = cfc_write_array_to_buffer( s, ptr, size );
+	if( ret != size )
+	{
+		dt282x_ai_cancel(dev,s);
 		return;
 	}
-	dt282x_copy_to_buffer(dev, ptr, size);
-	comedi_buf_write_free(s->async, size);
-
 	devpriv->nread-=size/2;
 
 	if(devpriv->nread<0){
@@ -493,20 +508,12 @@ static void dt282x_ai_dma_interrupt(comedi_device * dev)
 		devpriv->nread=0;
 	}
 	if (!devpriv->nread) {
-		devpriv->adcsr=0;
-		update_adcsr(0);
-
-		/* this eliminates "extra sample" issues */
-		devpriv->supcsr = 0;
-		update_supcsr(DT2821_ADCINIT);
-
+		dt282x_ai_cancel(dev, s);
 		s->async->events |= COMEDI_CB_EOA;
-		comedi_event(dev,s,s->async->events);
-
 		return;
 	}
 
-#if 1
+#if 0
 	/* clear the dual dma flag, making this the last dma segment */
 	/* XXX probably wrong */
 	if(!devpriv->ntrig){
@@ -516,13 +523,9 @@ static void dt282x_ai_dma_interrupt(comedi_device * dev)
 #endif
 	/* restart the channel */
 	prep_ai_dma(dev,i,0);
-
-	enable_dma(devpriv->dma[i].chan);
-
-	comedi_event(dev,s,s->async->events);
 }
 
-static int prep_ai_dma(comedi_device * dev,int chan,int n)
+static int prep_ai_dma(comedi_device * dev,int dma_index,int n)
 {
 	int dma_chan;
 	unsigned long dma_ptr;
@@ -533,39 +536,45 @@ static int prep_ai_dma(comedi_device * dev,int chan,int n)
 
 	if(n==0)
 		n = devpriv->dma_maxsize;
-	if (n >= devpriv->ntrig*2)
+	if (n > devpriv->ntrig*2)
 		n = devpriv->ntrig*2;
 	devpriv->ntrig -= n/2;
 
-	devpriv->dma[chan].size = n;
-	dma_chan = devpriv->dma[chan].chan;
-	dma_ptr = virt_to_bus(devpriv->dma[chan].buf);
+	devpriv->dma[dma_index].size = n;
+	dma_chan = devpriv->dma[dma_index].chan;
+	dma_ptr = virt_to_bus(devpriv->dma[dma_index].buf);
 
 	set_dma_mode(dma_chan, DMA_MODE_READ);
 	flags=claim_dma_lock();
+	clear_dma_ff(dma_chan);
 	set_dma_addr(dma_chan, dma_ptr);
 	set_dma_count(dma_chan, n);
 	release_dma_lock(flags);
 
+	enable_dma(dma_chan);
+
 	return n;
 }
 
-static int prep_ao_dma(comedi_device * dev,int chan,int n)
+static int prep_ao_dma(comedi_device * dev,int dma_index,int n)
 {
 	int dma_chan;
 	unsigned long dma_ptr;
 	unsigned long flags;
 
-	devpriv->dma[chan].size = n;
-	dma_chan = devpriv->dma[chan].chan;
-	dma_ptr = virt_to_bus(devpriv->dma[chan].buf);
+	devpriv->dma[dma_index].size = n;
+	dma_chan = devpriv->dma[dma_index].chan;
+	dma_ptr = virt_to_bus(devpriv->dma[dma_index].buf);
 
 	set_dma_mode(dma_chan, DMA_MODE_WRITE);
 	flags=claim_dma_lock();
+	clear_dma_ff(dma_chan);
 	set_dma_addr(dma_chan, dma_ptr);
 	set_dma_count(dma_chan, n );
 	release_dma_lock(flags);
 
+	enable_dma(dma_chan);
+
 	return n;
 }
 
@@ -575,27 +584,27 @@ static irqreturn_t dt282x_interrupt(int irq, void *d, struct pt_regs *regs)
 	comedi_subdevice *s = dev->subdevices+0;
 	comedi_subdevice *s_ao = dev->subdevices+1;
 	unsigned int supcsr, adcsr, dacsr;
-	sampl_t data;
-	int ret;
+	int handled = 0;
 
-	adcsr=inw(dev->iobase + DT2821_ADCSR);
-	if (adcsr & DT2821_ADERR) {
-		comedi_error(dev, "A/D error");
-		dt282x_ai_cancel(dev,s);
-		s->async->events |= COMEDI_CB_ERROR;
-		comedi_event(dev,s,s->async->events);
-		return IRQ_HANDLED;
-	}
+	adcsr = inw(dev->iobase + DT2821_ADCSR);
+	dacsr = inw(dev->iobase + DT2821_DACSR);
 	supcsr = inw(dev->iobase + DT2821_SUPCSR);
-	/*printk("supcsr=%02x\n",supcsr);*/
 	if (supcsr & DT2821_DMAD) {
 		if(devpriv->dma_dir==DMA_MODE_READ)
 			dt282x_ai_dma_interrupt(dev);
 		else
 			dt282x_ao_dma_interrupt(dev);
-		return IRQ_HANDLED;
+		handled = 1;
 	}
-	if ((dacsr = inw(dev->iobase + DT2821_DACSR)) & DT2821_DAERR) {
+	if (adcsr & DT2821_ADERR) {
+		if(devpriv->nread != 0 ){
+			comedi_error(dev, "A/D error");
+			dt282x_ai_cancel(dev,s);
+			s->async->events |= COMEDI_CB_ERROR;
+		}
+		handled = 1;
+	}
+	if (dacsr & DT2821_DAERR) {
 #if 0
 		static int warn = 5;
 		if(--warn<=0){
@@ -606,10 +615,13 @@ static irqreturn_t dt282x_interrupt(int irq, void *d, struct pt_regs *regs)
 		comedi_error(dev, "D/A error");
 		dt282x_ao_cancel(dev,s_ao);
 		s->async->events |= COMEDI_CB_ERROR;
-		comedi_event(dev,s_ao,s->async->events);
-		return IRQ_HANDLED;
+		handled = 1;
 	}
+#if 0
 	if (adcsr & DT2821_ADDONE) {
+		int ret;
+		sampl_t data;
+
 		data = (sampl_t) inw(dev->iobase + DT2821_ADDAT);
 		data&=(1<<boardtype.adbits)-1;
 		if(devpriv->ad_2scomp){
@@ -627,11 +639,12 @@ static irqreturn_t dt282x_interrupt(int irq, void *d, struct pt_regs *regs)
 			if(supcsr&DT2821_SCDN)
 				update_supcsr(DT2821_STRIG);
 		}
-
-		comedi_event(dev,s,s->async->events);
-		return IRQ_HANDLED;
+		handled = 1;
 	}
-	return IRQ_HANDLED;
+#endif
+	comedi_event(dev, s, s->async->events);
+	/* printk("adcsr=0x%02x dacsr-0x%02x supcsr=0x%02x\n", adcsr, dacsr, supcsr); */
+	return IRQ_RETVAL(handled);
 }
 
 
@@ -661,6 +674,7 @@ static int dt282x_ai_insn_read(comedi_device *dev,comedi_subdevice *s,
 {
 	int i;
 
+	/* XXX should we really be enabling the ad clock here? */
 	devpriv->adcsr = DT2821_ADCLK;
 	update_adcsr(0);
 
@@ -756,6 +770,10 @@ static int dt282x_ai_cmdtest(comedi_device * dev, comedi_subdevice * s,comedi_cm
 		cmd->convert_arg=SLOWEST_TIMER;
 		err++;
 	}
+	if(cmd->convert_arg < this_board->ai_speed ){
+		cmd->convert_arg = this_board->ai_speed;
+		err++;
+	}
 	if(cmd->scan_end_arg!=cmd->chanlist_len){
 		cmd->scan_end_arg=cmd->chanlist_len;
 		err++;
@@ -788,6 +806,15 @@ static int dt282x_ai_cmd(comedi_device * dev, comedi_subdevice * s)
 	comedi_cmd *cmd=&s->async->cmd;
 	int timer;
 
+	if(devpriv->usedma == 0){
+		comedi_error(dev, "driver requires 2 dma channels to execute command");
+		return -EIO;
+	}
+
+	dt282x_disable_dma(dev);
+
+	if(cmd->convert_arg < this_board->ai_speed )
+		cmd->convert_arg = this_board->ai_speed;
 	timer=dt282x_ns_to_timer(&cmd->convert_arg,TRIG_ROUND_NEAREST);
 	outw(timer, dev->iobase + DT2821_TMRCTR);
 
@@ -799,24 +826,20 @@ static int dt282x_ai_cmd(comedi_device * dev, comedi_subdevice * s)
 		devpriv->supcsr = DT2821_ERRINTEN | DT2821_DS0 | DT2821_DS1;
 	}
 	update_supcsr(DT2821_CLRDMADNE | DT2821_BUFFB | DT2821_ADCINIT);
-	devpriv->adcsr = 0;
 
 	devpriv->ntrig=cmd->stop_arg*cmd->scan_end_arg;
 	devpriv->nread=devpriv->ntrig;
 
 	devpriv->dma_dir=DMA_MODE_READ;
-	devpriv->current_dma_chan=0;
+	devpriv->current_dma_index=0;
 	prep_ai_dma(dev,0,0);
-	enable_dma(devpriv->dma[0].chan);
 	if(devpriv->ntrig){
 		prep_ai_dma(dev,1,0);
-		enable_dma(devpriv->dma[1].chan);
 		devpriv->supcsr |= DT2821_DDMA;
 		update_supcsr(0);
 	}
 
-	devpriv->adcsr = DT2821_ADCLK | DT2821_IADDONE;
-	update_adcsr(0);
+	devpriv->adcsr = 0;
 
 	dt282x_load_changain(dev,cmd->chanlist_len,cmd->chanlist);
 
@@ -839,8 +862,18 @@ static int dt282x_ai_cmd(comedi_device * dev, comedi_subdevice * s)
 	return 0;
 }
 
+static void dt282x_disable_dma(comedi_device *dev)
+{
+	if( devpriv->usedma ){
+		disable_dma(devpriv->dma[0].chan);
+		disable_dma(devpriv->dma[1].chan);
+	}
+}
+
 static int dt282x_ai_cancel(comedi_device * dev, comedi_subdevice * s)
 {
+	dt282x_disable_dma(dev);
+
 	devpriv->adcsr=0;
 	update_adcsr(0);
 
@@ -1016,7 +1049,7 @@ static int dt282x_ao_inttrig(comedi_device *dev,comedi_subdevice *s,
 
 	if(x!=0)return -EINVAL;
 
-	size = comedi_buf_read_n_available(s->async);
+	size = comedi_buf_read_n_available(s);
 	if(size>devpriv->dma_maxsize)size=devpriv->dma_maxsize;
 	if( size == 0){
 		rt_printk("dt282x: AO underrun\n");
@@ -1025,9 +1058,8 @@ static int dt282x_ao_inttrig(comedi_device *dev,comedi_subdevice *s,
 	comedi_buf_memcpy_from(s->async, 0, devpriv->dma[0].buf, size);
 	comedi_buf_read_free(s->async, size);
 	prep_ao_dma(dev,0,size);
-	enable_dma(devpriv->dma[0].chan);
 
-	size = comedi_buf_read_n_available(s->async);
+	size = comedi_buf_read_n_available(s);
 	if(size>devpriv->dma_maxsize)size=devpriv->dma_maxsize;
 	if( size == 0){
 		rt_printk("dt282x: AO underrun\n");
@@ -1036,7 +1068,6 @@ static int dt282x_ao_inttrig(comedi_device *dev,comedi_subdevice *s,
 	comedi_buf_memcpy_from(s->async, 0, devpriv->dma[1].buf, size);
 	comedi_buf_read_free(s->async, size);
 	prep_ao_dma(dev,1,size);
-	enable_dma(devpriv->dma[1].chan);
 
 	update_supcsr(DT2821_STRIG);
 	s->async->inttrig=NULL;
@@ -1049,6 +1080,13 @@ static int dt282x_ao_cmd(comedi_device *dev,comedi_subdevice *s)
 	int timer;
 	comedi_cmd *cmd=&s->async->cmd;
 
+	if(devpriv->usedma == 0){
+		comedi_error(dev, "driver requires 2 dma channels to execute command");
+		return -EIO;
+	}
+
+	dt282x_disable_dma(dev);
+
 	devpriv->supcsr = DT2821_ERRINTEN | DT2821_DS1 | DT2821_DDMA;
 	update_supcsr(DT2821_CLRDMADNE | DT2821_BUFFB | DT2821_DACINIT);
 
@@ -1056,7 +1094,7 @@ static int dt282x_ao_cmd(comedi_device *dev,comedi_subdevice *s)
 	devpriv->nread=devpriv->ntrig;
 
 	devpriv->dma_dir=DMA_MODE_WRITE;
-	devpriv->current_dma_chan=0;
+	devpriv->current_dma_index=0;
 
 	timer=dt282x_ns_to_timer(&cmd->scan_begin_arg,TRIG_ROUND_NEAREST);
 	outw(timer, dev->iobase + DT2821_TMRCTR);
@@ -1071,6 +1109,8 @@ static int dt282x_ao_cmd(comedi_device *dev,comedi_subdevice *s)
 
 static int dt282x_ao_cancel(comedi_device * dev, comedi_subdevice * s)
 {
+	dt282x_disable_dma(dev);
+
 	devpriv->dacsr=0;
 	update_dacsr(0);
 
