@@ -704,16 +704,28 @@ static int dt3k_dio_insn_config(comedi_device *dev,comedi_subdevice *s,
 {
 	int mask;
 
-	if(insn->n!=1)return -EINVAL;
-
 	mask=(CR_CHAN(insn->chanspec)<4)?0x0f:0xf0;
-	if(data[0]==COMEDI_OUTPUT)s->io_bits|=mask;
-	else s->io_bits&=~mask;
-
+	
+	switch(data[0])
+	{
+	case INSN_CONFIG_DIO_OUTPUT:
+		s->io_bits|=mask;
+		break;
+	case INSN_CONFIG_DIO_INPUT:
+		s->io_bits&=~mask;
+		break;
+	case INSN_CONFIG_DIO_QUERY:
+		data[1] = (s->io_bits & (1 << CR_CHAN(insn->chanspec))) ? COMEDI_OUTPUT : COMEDI_INPUT;
+		return insn->n;
+		break;
+	default:
+		return -EINVAL;
+		break;
+	}	
 	mask=(s->io_bits&0x01)|((s->io_bits&0x10)>>3);
 	dt3k_dio_config(dev,mask);
 
-	return 1;
+	return insn->n;
 }
 
 static int dt3k_dio_insn_bits(comedi_device *dev,comedi_subdevice *s,
@@ -841,6 +853,8 @@ static int dt3000_detach(comedi_device *dev)
 {
 	if(dev->irq)comedi_free_irq(dev->irq,dev);
 
+	if(devpriv && devpriv->pci_dev) pci_dev_put(devpriv->pci_dev);
+
 	/* XXX */
 
 	return 0;
@@ -883,63 +897,26 @@ static int setup_pci(comedi_device *dev)
 	printk("0x%08lx mapped to %p, ",devpriv->phys_addr,devpriv->io_addr);
 #endif
 
-	dev->iobase = (int)devpriv->io_addr;
+	dev->iobase = (unsigned long)devpriv->io_addr;
 
 	return 0;
 }
 
-#if LINUX_VERSION_CODE < 0x020300
 static struct pci_dev *dt_pci_find_device(struct pci_dev *from,int *board)
 {
 	int i;
 	
-	if(!from){
-		from=pci_devices;
-	}else{
-		from=from->next;
-	}
-	while(from){
-		if(from->vendor == PCI_VENDOR_ID_DT){
-			for(i=0;i<n_dt3k_boards;i++){
-				if(from->device == dt3k_boardtypes[i].device_id){
-					*board=i;
-					return from;
-				}
+	for(from=pci_get_device(PCI_VENDOR_ID_DT,PCI_ANY_ID,from); from!=NULL;
+			from=pci_get_device(PCI_VENDOR_ID_DT,PCI_ANY_ID,from)){
+		for(i=0;i<n_dt3k_boards;i++){
+			if(from->device == dt3k_boardtypes[i].device_id){
+				*board=i;
+				return from;
 			}
-			printk("unknown Data Translation PCI device found with device_id=0x%04x\n",from->device);
 		}
-		from=from->next;
+		printk("unknown Data Translation PCI device found with device_id=0x%04x\n",from->device);
 	}
 	*board=-1;
 	return from;
 }
 
-#else
-
-static struct pci_dev *dt_pci_find_device(struct pci_dev *from,int *board)
-{
-	int i;
-	
-	if(!from){
-		from=(struct pci_dev *)(pci_devices.next);
-	}else{
-		from=(struct pci_dev *)(from->global_list.next);
-	}
-	while(from){
-		if(from->vendor == PCI_VENDOR_ID_DT){
-			for(i=0;i<n_dt3k_boards;i++){
-				if(from->device == dt3k_boardtypes[i].device_id){
-					*board=i;
-					return from;
-				}
-			}
-			printk("unknown Data Translation PCI device found with device_id=0x%04x\n",from->device);
-		}
-		from=(struct pci_dev *)(from->global_list.next);
-	}
-	*board=-1;
-	return from;
-}
-
-#endif
-
