@@ -28,8 +28,8 @@
 /*
 
 Driver: gsc_hpdi.o
-Description: Driver for the General Standards Corporation High
-    Speed Parallel Digital Interface rs485 boards.
+Description: General Standards Corporation High
+    Speed Parallel Digital Interface rs485 boards
 Author: Frank Mori Hess <fmhess@users.sourceforge.net>
 Status: only receive mode works, transmit not supported
 Updated: 2003-02-20
@@ -346,18 +346,19 @@ static int dio_config_insn( comedi_device *dev,comedi_subdevice *s,
 {
 	switch( data[ 0 ] )
 	{
-		case COMEDI_OUTPUT:
-			if( insn->n != 1 ) return -EINVAL;
+		case INSN_CONFIG_DIO_OUTPUT:
 			priv(dev)->dio_config_output = 1;
-			return 1;
+			return insn->n;
 			break;
-		case COMEDI_INPUT:
-			if( insn->n != 1 ) return -EINVAL;
+		case INSN_CONFIG_DIO_INPUT:
 			priv(dev)->dio_config_output = 0;
-			return 1;
+			return insn->n;
+			break;
+		case INSN_CONFIG_DIO_QUERY:
+			data[1] = priv(dev)->dio_config_output ? COMEDI_OUTPUT : COMEDI_INPUT;
+			return insn->n;
 			break;
 		case INSN_CONFIG_BLOCK_SIZE:
-			if( insn->n != 2 ) return -EINVAL;
 			return dio_config_block_size( dev, data );
 			break;
 		default:
@@ -548,7 +549,7 @@ static int hpdi_attach(comedi_device *dev, comedi_devconfig *it)
 	{
 		do
 		{
-			pcidev = pci_find_subsys( PCI_VENDOR_ID_PLX, hpdi_boards[ i ].device_id,
+			pcidev = pci_get_subsys( PCI_VENDOR_ID_PLX, hpdi_boards[ i ].device_id,
 				PCI_VENDOR_ID_PLX, hpdi_boards[ i ].subdevice_id, pcidev );
 			// was a particular bus/slot requested?
 			if( it->options[0] || it->options[1] )
@@ -575,7 +576,10 @@ static int hpdi_attach(comedi_device *dev, comedi_devconfig *it)
 		pcidev->bus->number, PCI_SLOT(pcidev->devfn));
 
 	if( pci_enable_device( pcidev ) )
+	{
+		pci_dev_put( pcidev );
 		return -EIO;
+	}
 	pci_set_master( pcidev );
 
 	priv(dev)->hw_dev = pcidev;
@@ -678,9 +682,9 @@ static int hpdi_detach(comedi_device *dev)
 				pci_free_consistent( priv(dev)->hw_dev, sizeof( struct plx_dma_desc ) *
 				NUM_DMA_DESCRIPTORS, priv(dev)->dma_desc, priv(dev)->dma_desc_phys_addr );
 			pci_disable_device( priv(dev)->hw_dev );
+			pci_dev_put(priv(dev)->hw_dev);
 		}
 	}
-
 	return 0;
 }
 
