@@ -268,7 +268,7 @@ static int icp_multi_insn_read_ai(comedi_device * dev, comedi_subdevice * s, com
 	setup_channel_list(dev, s, &insn->chanspec, 1);
 
 #ifdef ICP_MULTI_EXTDEBUG
-	printk("icp_multi A ST=%4x IO=%x\n",readw(dev->iobase+ICP_MULTI_ADC_CSR), dev->iobase+ICP_MULTI_ADC_CSR);
+	printk("icp_multi A ST=%4x IO=%lx\n",readw(dev->iobase+ICP_MULTI_ADC_CSR), dev->iobase+ICP_MULTI_ADC_CSR);
 #endif
 
 	for (n=0; n<insn->n; n++) {
@@ -919,7 +919,7 @@ static int icp_multi_attach(comedi_device *dev,comedi_devconfig *it)
 	printk("0x%08lx mapped to %p, ", iobase, devpriv->io_addr);
 #endif
 
-	dev->iobase = (int)devpriv->io_addr;
+	dev->iobase = (unsigned long)devpriv->io_addr;
 
 	dev->board_name = this_board->name;
 
