@@ -22,8 +22,8 @@
  */
 /*
 Driver: me4000.o
-Description: Source code for the Meilhaus ME-4000 board family.
-Devices: ME-4650, ME-4670i, ME-4680, ME-4680i, ME-4680is
+Description: Meilhaus ME-4000 series boards
+Devices: [Meilhaus] ME-4650 (me4000), ME-4670i, ME-4680, ME-4680i, ME-4680is
 Author: gg (Guenter Gebhardt <g.gebhardt@meilhaus.com>)
 Updated: Mon, 18 Mar 2002 15:34:01 -0800
 Status: broken (no support for loading firmware)
@@ -42,6 +42,13 @@ Configuration Options:
 
     If bus/slot is not specified, the first available PCI
     device will be used.
+
+The firmware required by these boards is available in the
+comedi_nonfree_firmware tarball available from
+http://www.comedi.org.  However, the driver's support for
+loading the firmware through comedi_config is currently
+broken.
+
  */
 
 #include <linux/comedidev.h>
@@ -397,7 +404,8 @@ static int me4000_probe(comedi_device *dev, comedi_devconfig *it){
     /*
      * Probe the device to determine what device in the series it is.
      */
-    pci_for_each_dev(pci_device){
+ 	for(pci_device = pci_get_device(PCI_ANY_ID, PCI_ANY_ID, NULL); pci_device != NULL ; 
+		pci_device = pci_get_device(PCI_ANY_ID, PCI_ANY_ID, pci_device)) {
 	if(pci_device->vendor == PCI_VENDOR_ID_MEILHAUS){
 	    for(i = 0; i < ME4000_BOARD_VERSIONS; i++){
 		if(me4000_boards[i].device_id == pci_device->device){
@@ -429,8 +437,10 @@ found:
 	    pci_device->bus->number, PCI_SLOT(pci_device->devfn));
 
     /* Allocate private memory */
-    if(alloc_private(dev, sizeof(me4000_info_t)) < 0)
+    if(alloc_private(dev, sizeof(me4000_info_t)) < 0){
+	pci_dev_put(pci_device);
 	return -ENOMEM;
+    }
 
     /* Set data in device structure */
     dev->board_name = board->name;
@@ -505,7 +515,6 @@ PROBE_ERROR_2:
     pci_release_regions(pci_device);
 
 PROBE_ERROR_1:
-    kfree(info);
 
     return result;
 }
@@ -846,11 +855,14 @@ static int reset_board(comedi_device *dev){
 static int me4000_detach(comedi_device *dev){
     CALL_PDEBUG("In me4000_detach()\n");
 
-    reset_board(dev);
+    if(info){
+	reset_board(dev);
 
-    pci_release_regions(info->pci_dev_p);
-
-    kfree(info);
+	if(info->pci_dev_p) {
+	    pci_release_regions(info->pci_dev_p);
+	    pci_dev_put(info->pci_dev_p);
+	}
+    }
 
     return 0;
 }
@@ -1951,10 +1963,11 @@ static int me4000_dio_insn_config(
 
 	CALL_PDEBUG("In me4000_dio_insn_config()\n");
 
-
-	/* Only data[0] is valid */
-	if(insn->n != 1)
-	return -EINVAL;
+	if(data[0] == INSN_CONFIG_DIO_QUERY)
+	{
+		data[1] = (s->io_bits & (1 << chan)) ? COMEDI_OUTPUT : COMEDI_INPUT;
+		return insn->n;
+	}
 
     /* 
      * The input or output configuration of each digital line is
