@@ -25,7 +25,7 @@
 
 /*
 Driver: me_daq.o
-Description: Driver for the Meilhaus PCI data acquisition cards.
+Description: Meilhaus PCI data acquisition cards
 Author: Michael Hillmann <hillmann@syscongroup.de>
 Devices: [Meilhaus] ME-2600i (me_daq), ME-2000i
 Status: experimental
@@ -41,6 +41,13 @@ Configuration options:
 
     If bus/slot is not specified, the first available PCI
     device will be used.
+
+The 2600 requires a firmware upload, which can be accomplished
+using the -i or --init-data option of comedi_config. 
+The firmware can be
+found in the comedi_nonfree_firmware tarball available
+from http://www.comedi.org 
+
 */
 
 #include <linux/comedidev.h>
@@ -261,8 +268,8 @@ COMEDI_INITCLEANUP(me_driver);
 typedef struct
 {
   struct pci_dev* pci_device;
-  unsigned int plx_regbase;         // PLX configuration base address
-  unsigned int me_regbase;          // Base address of the Meilhaus card
+  unsigned long plx_regbase;         // PLX configuration base address
+  unsigned long me_regbase;          // Base address of the Meilhaus card
   unsigned int plx_regbase_size;    // Size of PLX configuration space
   unsigned int me_regbase_size;     // Size of Meilhaus space
 
@@ -684,7 +691,8 @@ static int me_attach(comedi_device *dev,comedi_devconfig *it)
 //
 // Probe the device to determine what device in the series it is.
 //
-  pci_for_each_dev(pci_device)
+	for(pci_device = pci_get_device(PCI_ANY_ID, PCI_ANY_ID, NULL); pci_device != NULL ; 
+		pci_device = pci_get_device(PCI_ANY_ID, PCI_ANY_ID, pci_device))
   {
     if(pci_device->vendor == PCI_VENDOR_ID_MEILHAUS)
     {
@@ -724,7 +732,10 @@ found:
   // Allocate private memory
 
   if(alloc_private(dev,sizeof(me_private_data_struct)) < 0)
+  {
+    pci_dev_put(pci_device);
     return -ENOMEM;
+  }
 
   // Set data in device structure
 
@@ -806,7 +817,7 @@ found:
     unsigned char *aux_data;
     int aux_len;
 
-    aux_data = (unsigned char *)it->options[COMEDI_DEVCONF_AUX_DATA];
+    aux_data = comedi_aux_data(it->options, 0);
     aux_len = it->options[COMEDI_DEVCONF_AUX_DATA_LENGTH];
 
     if(!aux_data || aux_len)return -EINVAL;
@@ -865,7 +876,15 @@ found:
 
 static int me_detach(comedi_device *dev)
 {
-  me_reset(dev);
+  if(dev_private)
+  {
+    me_reset(dev);
+
+    if(dev_private->pci_device)
+    {
+      pci_dev_put(dev_private->pci_device);
+    }
+  }
 
   return 0;
 }
