@@ -35,10 +35,6 @@ supported yet.  PulseGeneration (both single pulse and pulse train)
 works. DIO is experimental (8 channels only). Interrupts do not
 work.
 
-The counter API is unstable, see comedi_counter_unstable.h and
-the directory comedi/Documentation/comedi/ (the files counter-spec and
-insn_config).
-
 References:
 DAQ 660x Register-Level Programmer Manual  (NI 370505A-01)
 DAQ 6601/6602 User Manual (NI 322137B-01)
@@ -53,7 +49,6 @@ Measurement, ...)
 
 #include <linux/comedidev.h>
 #include "mite.h"
-#include "comedi_counter_unstable.h"
 
 #define CTRS_PER_CHIP 4 // The number of counters per ni-tio chip
 #define DATA_1B 0x1 // 1 byte = 8 bits data
@@ -519,7 +514,12 @@ NI_660x_GPCT_Config ni_660x_gpct_config[NI_660X_GPCT_MAXCHANNELS];
 #define SourceSelectTimebase1	0x0<<2
 #define SourceSelectTimebase2	0x12<<2
 #define SourceSelectTimebase3	0x1e<<2
+#define SourceSelectSourcePinI  0x1<<2
+#define SourceSelectSourcePin0  0x2<<2
 #define GateSelectSource        0x0<<7
+#define GateSelectGatePinI      0x1<<7
+#define GateSelectLogicLow      0x30<<7
+
 
 #define TriggerModeStartStop	0x0<<3
 #define TriggerModeStopStart	0x1<<3
@@ -651,6 +651,7 @@ typedef struct
 
 static int ni_660x_attach(comedi_device *dev,comedi_devconfig *it);
 static int ni_660x_detach(comedi_device *dev);
+static void init_tio_chip(comedi_device *dev, int chipset);
 
 static comedi_driver driver_ni_660x=
 {
@@ -777,6 +778,16 @@ static int ni_660x_attach(comedi_device *dev,comedi_devconfig *it)
 	s->insn_config  = ni_660x_dio_insn_config;
 	s->io_bits      = 0;     /* all bits default to input */
 
+	init_tio_chip(dev, 0);
+	// Enabling the second chip:  This "hardcodes" the counter
+	// outputs 5 to 8 to use the second TIO in case of a NI6602
+	// board.
+	if (thisboard->name == "PCI-6602" )
+	{
+		printk("NI6602: Setting Counterswap on second TIO\n");
+		init_tio_chip(dev, 1);
+	}
+
 	printk("attached\n");
 
 	/* What does this "return value" mean?  Is this fixed by API??
@@ -812,7 +823,6 @@ static int GPCT_check_chipset_from_channel(comedi_device *dev, int channel)
 	}else if ( (channel >= CTRS_PER_CHIP) && (channel < thisboard->n_ctrs) )
 	{
 		chipset = 1;
-		// DPRINTK("NI_660x: Moving to chipset 1\n");
 	}else
 	{
 		DPRINTK("NI_660x: Channel specification not between limits\n");
@@ -840,16 +850,14 @@ ni_660x_GPCT_rinsn(comedi_device *dev, comedi_subdevice *s,
 		value is
 	*/
 	int tmpdata[2];
-	int address;
+	unsigned long address;
 
 	/* ============================================================ */
 	/* 1 subdevice with 8 channels, differentation based on channel */
-	DPRINTK("NI_660x: INSN_READ on channel %d\n", subdev_channel);
-
 	// Check what Application of Counter this channel is configured for
 	switch(ni_660x_gpct_config[subdev_channel].App)
 	{
-	case PositionMeasurement:
+	case PositionMeasurement: case CountingAndTimeMeasurement: 
 		// Check if (n > 0)
 		if ( insn->n <= 0 )
 		{
@@ -887,25 +895,20 @@ ni_660x_GPCT_rinsn(comedi_device *dev, comedi_subdevice *s,
     }
 }
 
-static void
-enable_chip(comedi_device *dev, int chipset)
+static void init_tio_chip(comedi_device *dev, int chipset)
 {
-	/* See P. 3.5 of the Register-Level Programming manual.  This
-		bit has to be set, otherwise, you can't use the second chip.
+	/* See P. 3.5 of the Register-Level Programming manual.  The
+		CounterSwap bit has to be set on the second chip, otherwise 
+		it will try to use the same pins as the first chip.
 	*/
-	if ( chipset == 1)
-	{
-		writel(CounterSwap,dev->iobase + GPCT_OFFSET[chipset]
-		+ registerData[ClockConfigRegister].offset);
-	}else
-	{
-		writel(0x0,dev->iobase + GPCT_OFFSET[chipset]
-		+ registerData[ClockConfigRegister].offset);
-	}
+	if(chipset)
+		writel(CounterSwap,dev->iobase + GPCT_OFFSET[1]
+			+ registerData[ClockConfigRegister].offset);
+	else
+		writel(0,dev->iobase + GPCT_OFFSET[0]
+			+ registerData[ClockConfigRegister].offset);	
 }
 
-
-
 static int
 ni_660x_GPCT_insn_config(comedi_device *dev, comedi_subdevice *s,
 	comedi_insn *insn, lsampl_t *data)
@@ -915,7 +918,6 @@ ni_660x_GPCT_insn_config(comedi_device *dev, comedi_subdevice *s,
 	int counter_channel = GPCT_check_counter_channel_from_subdev_channel(subdev_channel);
 
 	DPRINTK("NI_660x: INSN_CONFIG: Configuring Channel %d\n", subdev_channel);
-	enable_chip(dev, chipset);
 
 	// Check what type of Counter the user requested, data[0] contains
 	// the Application type
@@ -979,13 +981,13 @@ ni_660x_GPCT_insn_config(comedi_device *dev, comedi_subdevice *s,
 		// Load (latch) this value into the counter
 		writew(Load,dev->iobase + GPCT_OFFSET[chipset]
 			+ registerData[GxCommandRegister(counter_channel)].offset);
-		/* - Set Counting Mode into GPCT_X1 / 2 / 4 (as set by user
-		- Take into account Z pulse (index pulse) only when both
-		channel A and B are high (arbitrary choice)
+		/* - Set Counting Mode into GPCT_X1 / 2 / 4 (as set by user)
+		   - When to take into account index pulse (as set by user)
+		   - Take into account index pulse (as set by user)
 		*/
 		writew(((ni_660x_gpct_config[subdev_channel]).data[0] |
 			(ni_660x_gpct_config[subdev_channel]).data[1] |
-			(ni_660x_gpct_config[subdev_channel]).data[1] ),
+			(ni_660x_gpct_config[subdev_channel]).data[2] ),
 			dev->iobase + GPCT_OFFSET[chipset]
 			+ registerData[GxCountingModeRegister(counter_channel)].offset);
 		// Put counter in input mode
@@ -1120,6 +1122,49 @@ ni_660x_GPCT_insn_config(comedi_device *dev, comedi_subdevice *s,
 		writew(Arm|UpDownDown,dev->iobase + GPCT_OFFSET[chipset]
 			+ registerData[GxCommandRegister(counter_channel)].offset);
 		break;
+	case GPCT_SIMPLE_EVENT:
+		DPRINTK("NI_660x: INSN_CONFIG: Config Simple Event Counter\n");
+		ni_660x_gpct_config[subdev_channel].App = 
+			CountingAndTimeMeasurement;
+		// Reset the counter
+		writew(GxReset(counter_channel),
+			dev->iobase + GPCT_OFFSET[chipset] + registerData[
+				GxxJointResetRegister(counter_channel)].offset);
+		// Disarm
+		writew(Disarm,
+			dev->iobase + GPCT_OFFSET[chipset] + registerData[
+				GxCommandRegister(counter_channel)].offset);
+		// Put 0 as initial counter value in the load register
+		writel(0x0,
+			dev->iobase + GPCT_OFFSET[chipset] + registerData[
+				GxLoadARegister(counter_channel)].offset);
+		// Load (latch) this value into the counter
+		writew(Load,
+			dev->iobase + GPCT_OFFSET[chipset] + registerData[
+				GxCommandRegister(counter_channel)].offset);
+		// Set gate logic low, & select source pin dedicated to channel
+		writew(GateSelectLogicLow|SourceSelectSourcePinI,
+			dev->iobase + GPCT_OFFSET[chipset] + registerData[
+		 		GxInputSelectRegister(counter_channel)].offset);
+		// Disable gate for simple event counting
+		writew(GatingModeDisabled,
+			dev->iobase + GPCT_OFFSET[chipset] + registerData[
+				GxModeRegister(counter_channel)].offset);
+		// Use normal counting mode (instead of synchronous)
+		writew(CountingModeNormal,
+			dev->iobase + GPCT_OFFSET[chipset] + registerData[
+				GxCountingModeRegister(counter_channel)].offset
+		);
+		// Put counter in input mode
+		// Not necessary since this is the default ...
+		/* writel(Counter_A_Is_Input, //NOT WORKING -- REFER KG's file.
+		 * 	dev->iobase + GPCT_OFFSET[chipset] + registerData[
+		 * 		IOConfigReg36_39].offset); */
+		// Arm the counter and put it into always counting up mode
+		writew(UpDownUp|Arm,
+			dev->iobase + GPCT_OFFSET[chipset] + registerData[
+				GxCommandRegister(counter_channel)].offset);
+		break;
 	default:
 		DPRINTK("NI_660x: unsupported insn_config\n");
 		return -EINVAL;
@@ -1213,7 +1258,6 @@ static int ni_660x_GPCT_inttrig(comedi_device *dev,
 	int counter_channel = GPCT_check_counter_channel_from_subdev_channel(subdev_channel);
 
 	DPRINTK("Triggering channel %d\n", subdev_channel);
-  enable_chip(dev, chipset);
 
 	// Reset the counter
 	writew(GxReset(counter_channel),dev->iobase + GPCT_OFFSET[chipset]
@@ -1278,31 +1322,34 @@ static int ni_660x_dio_insn_config(comedi_device *dev,
 				   comedi_insn *insn,
 				   lsampl_t *data)
 {
-  int chan=CR_CHAN(insn->chanspec);
-
-  if(insn->n!=1)return -EINVAL;
-
-  /* The input or output configuration of each digital line is
-   * configured by a special insn_config instruction.  chanspec
-   * contains the channel to be changed, and data[0] contains the
-   * value COMEDI_INPUT or COMEDI_OUTPUT. */
-
-  if(data[0]==COMEDI_OUTPUT)
-    {
-      s->io_bits |= 1<<chan;
-    }
-  else
-    {
-      s->io_bits &= ~(1<<chan);
-    }
-  // No GPCT_OFFSET[chipset] offset here??
-  writew(s->io_bits,dev->iobase + registerData[STCDIOControl].offset);
-  /* Should we do also something with the IO configuration registers,
-     see p 3-38 of register level prog. manual
-  */
-
-  return 1;
-  return -EINVAL;
+	int chan=CR_CHAN(insn->chanspec);
+	
+	/* The input or output configuration of each digital line is
+	* configured by a special insn_config instruction.  chanspec
+	* contains the channel to be changed, and data[0] contains the
+	* value COMEDI_INPUT or COMEDI_OUTPUT. */
+	
+	switch(data[0])
+	{
+	case INSN_CONFIG_DIO_OUTPUT:
+		s->io_bits |= 1<<chan;
+		break;
+	case INSN_CONFIG_DIO_INPUT:
+		s->io_bits &= ~(1<<chan);
+		break;
+	case INSN_CONFIG_DIO_QUERY:
+		data[1] = (s->io_bits & (1 << chan)) ? COMEDI_OUTPUT : COMEDI_INPUT;
+		return insn->n;
+	default:
+		return -EINVAL;
+		break;
+	};
+	// No GPCT_OFFSET[chipset] offset here??
+	writew(s->io_bits,dev->iobase + registerData[STCDIOControl].offset);
+	/* Should we do also something with the IO configuration registers,
+		see p 3-38 of register level prog. manual
+	*/
+	return insn->n;
 }
 
 
