@@ -1,6 +1,6 @@
 /*
     ni_at_a2150.c driver for National Instruments AT-A2150 boards
-    Copyright (C) 2001 Frank Mori Hess <fmhess@uiuc.edu>
+    Copyright (C) 2001, 2002 Frank Mori Hess <fmhess@users.sourceforge.net>
 
     COMEDI - Linux Control and Measurement Device Interface
     Copyright (C) 2000 David A. Schleef <ds@schleef.org>
@@ -63,21 +63,13 @@ TRIG_WAKE_EOS
 
 */
 
-#include <linux/kernel.h>
-#include <linux/module.h>
-#include <linux/sched.h>
-#include <linux/mm.h>
-#include <linux/slab.h>
-#include <linux/errno.h>
-#include <linux/ioport.h>
-#include <linux/delay.h>
-#include <linux/interrupt.h>
-#include <linux/timex.h>
-#include <linux/timer.h>
-#include <asm/io.h>
-#include <asm/dma.h>
 #include <linux/comedidev.h>
+
+#include <linux/ioport.h>
+#include <asm/dma.h>
+
 #include "8253.h"
+#include "comedi_fc.h"
 
 #define A2150_SIZE           28
 #define A2150_DMA_BUFFER_SIZE	0xff00	// size in bytes of dma buffer
@@ -185,7 +177,7 @@ static comedi_driver driver_a2150={
 	detach:		a2150_detach,
 };
 
-static void a2150_interrupt(int irq, void *d, struct pt_regs *regs);
+static irqreturn_t a2150_interrupt(int irq, void *d, struct pt_regs *regs);
 static int a2150_ai_cmdtest(comedi_device *dev,comedi_subdevice *s,comedi_cmd *cmd);
 static int a2150_ai_cmd(comedi_device *dev, comedi_subdevice *s);
 static int a2150_ai_rinsn(comedi_device *dev, comedi_subdevice *s, comedi_insn *insn, lsampl_t *data);
@@ -210,7 +202,7 @@ static void ni_dump_regs(comedi_device *dev)
 #endif
 
 /* interrupt service routine */
-static void a2150_interrupt(int irq, void *d, struct pt_regs *regs)
+static irqreturn_t a2150_interrupt(int irq, void *d, struct pt_regs *regs)
 {
 	int i;
 	int status;
@@ -226,7 +218,7 @@ static void a2150_interrupt(int irq, void *d, struct pt_regs *regs)
 	if(dev->attached == 0)
 	{
 		comedi_error(dev, "premature interrupt");
-		return;
+		return IRQ_HANDLED;
 	}
 	// initialize async here to make sure s is not NULL
 	async = s->async;
@@ -238,7 +230,7 @@ static void a2150_interrupt(int irq, void *d, struct pt_regs *regs)
 	if((status & INTR_BIT ) == 0)
 	{
 		comedi_error(dev, "spurious interrupt");
-		return;
+		return IRQ_NONE;
 	}
 
 	if(status & OVFL_BIT)
@@ -254,7 +246,7 @@ static void a2150_interrupt(int irq, void *d, struct pt_regs *regs)
 		a2150_cancel(dev, s);
 		async->events |= COMEDI_CB_ERROR | COMEDI_CB_EOA;
 		comedi_event(dev, s, async->events);
-		return;
+		return IRQ_HANDLED;
 	}
 
 	flags = claim_dma_lock();
@@ -299,7 +291,7 @@ static void a2150_interrupt(int irq, void *d, struct pt_regs *regs)
 		dpnt = devpriv->dma_buffer[i];
 		// convert from 2's complement to unsigned coding
 		dpnt ^= 0x8000;
-		comedi_buf_put(async, dpnt);
+		cfc_write_to_buffer( s, dpnt );
 		if(cmd->stop_src == TRIG_COUNT)
 		{
 			if(--devpriv->count == 0)
@@ -311,9 +303,12 @@ static void a2150_interrupt(int irq, void *d, struct pt_regs *regs)
 		}
 	}
 	// re-enable  dma
-	set_dma_addr(devpriv->dma, virt_to_bus(devpriv->dma_buffer));
-	set_dma_count(devpriv->dma, leftover * sample_size);
-	enable_dma(devpriv->dma);
+	if(leftover)
+	{
+		set_dma_addr(devpriv->dma, virt_to_bus(devpriv->dma_buffer));
+		set_dma_count(devpriv->dma, leftover * sample_size);
+		enable_dma(devpriv->dma);
+	}
 	release_dma_lock(flags);
 
 	async->events |= COMEDI_CB_BLOCK;
@@ -323,7 +318,7 @@ static void a2150_interrupt(int irq, void *d, struct pt_regs *regs)
 	/* clear interrupt */
 	outw(0x00, dev->iobase + DMA_TC_CLEAR_REG);
 
-	return;
+	return IRQ_HANDLED;
 }
 
 // probes board type, returns offset
@@ -423,8 +418,7 @@ static int a2150_attach(comedi_device *dev, comedi_devconfig *it)
 	dev->board_ptr = a2150_boards + a2150_probe(dev);
 	dev->board_name = thisboard->name;
 
-	dev->n_subdevices = 1;
-	if(alloc_subdevices(dev) < 0)
+	if(alloc_subdevices(dev, 1) < 0)
 		return -ENOMEM;
 
 	/* analog input subdevice */
@@ -459,8 +453,7 @@ static int a2150_attach(comedi_device *dev, comedi_devconfig *it)
 	{
 		if((DCAL_BIT & inw(dev->iobase + STATUS_REG)) == 0)
 			break;
-		udelay(1000);
-	// probably should sleep instead of using udelay since wait is so long
+		comedi_udelay(1000);
 	}
 	if(i == timeout)
 	{
@@ -756,11 +749,11 @@ static int a2150_ai_cmd(comedi_device *dev, comedi_subdevice *s)
 	return 0;
 }
 
-// XXX doesn't seem to work
 static int a2150_ai_rinsn(comedi_device *dev, comedi_subdevice *s, comedi_insn *insn, lsampl_t *data)
 {
 	unsigned int i, n;
-	static const int timeout = 10000;
+	static const int timeout = 100000;
+	static const int filter_delay = 36;
 
 	// clear fifo and reset triggering circuitry
 	outw(0, dev->iobase + FIFO_RESET_REG);
@@ -786,9 +779,22 @@ static int a2150_ai_rinsn(comedi_device *dev, comedi_subdevice *s, comedi_insn *
 	// start aquisition for soft trigger
 	outw(0, dev->iobase + FIFO_START_REG);
 
-	/* there is a 35.6 sample delay for data to get through the antialias filter
-	 * so we might as well wait a while */
-	udelay(500);
+	/* there is a 35.6 sample delay for data to get through the antialias filter */
+ 	for(n = 0; n < filter_delay; n++)
+	{
+		for(i = 0; i < timeout; i++)
+		{
+			if(inw(dev->iobase + STATUS_REG) & FNE_BIT)
+				break;
+			comedi_udelay(1);
+		}
+		if(i == timeout)
+		{
+			comedi_error(dev, "timeout");
+			return -ETIME;
+		}
+		inw(dev->iobase + FIFO_DATA_REG);
+	}
 
 	// read data
  	for(n = 0; n < insn->n; n++)
@@ -797,6 +803,7 @@ static int a2150_ai_rinsn(comedi_device *dev, comedi_subdevice *s, comedi_insn *
 		{
 			if(inw(dev->iobase + STATUS_REG) & FNE_BIT)
 				break;
+			comedi_udelay(1);
 		}
 		if(i == timeout)
 		{
