@@ -26,7 +26,7 @@ Status: unknown
 Devices: [National Instruments] AT-MIO-16 (atmio16), AT-MIO-16D (atmio16d)
 */
 /*
- * I must give credit here to Michal Dobes <majkl@tesnet.cz> who
+ * I must give credit here to Michal Dobes <dobes@tesnet.cz> who
  * wrote the driver for Advantec's pcl812 boards. I used the interrupt 
  * handling code from his driver as an example for this one.
  * 
@@ -35,19 +35,10 @@ Devices: [National Instruments] AT-MIO-16 (atmio16), AT-MIO-16D (atmio16d)
  * 
  */
 
-#include <linux/kernel.h>
-#include <linux/module.h>
-#include <linux/sched.h>
-#include <linux/mm.h>
-#include <linux/slab.h>
-#include <linux/errno.h>
-#include <linux/ioport.h>
-#include <linux/delay.h>
-#include <linux/interrupt.h>
-#include <linux/timex.h>
-#include <linux/timer.h>
-#include <asm/io.h>
 #include <linux/comedidev.h>
+
+#include <linux/ioport.h>
+
 #include "8255.h"
 
 
@@ -136,7 +127,7 @@ static atmio16_board_t atmio16_boards[]={
 /* function prototypes */
 static int atmio16d_attach(comedi_device *dev,comedi_devconfig *it);
 static int atmio16d_detach(comedi_device *dev);
-static void atmio16d_interrupt(int irq, void *d, struct pt_regs *regs);
+static irqreturn_t atmio16d_interrupt(int irq, void *d, struct pt_regs *regs);
 static int atmio16d_ai_cmdtest(comedi_device *dev, comedi_subdevice *s, comedi_cmd *cmd);
 static int atmio16d_ai_cmd(comedi_device *dev, comedi_subdevice *s);
 static int atmio16d_ai_cancel(comedi_device *dev, comedi_subdevice *s);
@@ -263,28 +254,17 @@ static void reset_atmio16d(comedi_device *dev)
 	outw(2048, dev->iobase+DAC1_REG);
 }
 
-static void atmio16d_interrupt(int irq, void *d, struct pt_regs *regs)
+static irqreturn_t atmio16d_interrupt(int irq, void *d, struct pt_regs *regs)
 {
 	comedi_device *dev = d;
 	comedi_subdevice *s = dev->subdevices + 0;
-	
+
 //	printk("atmio16d_interrupt!\n");
 
-	*(sampl_t *)(s->async->data+s->async->buf_int_ptr) =
-		inw(dev->iobase+AD_FIFO_REG);
-	s->async->buf_int_ptr += sizeof(sampl_t);
-	s->async->buf_int_count += sizeof(sampl_t);
+	comedi_buf_put( s->async, inw(dev->iobase+AD_FIFO_REG) );
 
-	if((++s->async->cur_chan) >= s->async->cmd.chanlist_len) {	/* one scan done */
-		s->async->cur_chan = 0;
-		s->async->events |= COMEDI_CB_EOS;
-	}
-
-	if (s->async->buf_int_ptr >= s->async->data_len) {	/* buffer rollover */
-		s->async->buf_int_ptr = 0;
-		s->async->events |= COMEDI_CB_EOBUF;
-	}
 	comedi_event(dev, s, s->async->events);
+	return IRQ_HANDLED;
 }
 
 static int atmio16d_ai_cmdtest(comedi_device *dev, comedi_subdevice *s, comedi_cmd *cmd)
@@ -737,10 +717,7 @@ static int atmio16d_attach(comedi_device * dev, comedi_devconfig * it)
 	/* board name */
 	dev->board_name = boardtype->name;
 
-	/* set number of subdevices */
-	dev->n_subdevices=4;
-
-	if((ret=alloc_subdevices(dev))<0)
+	if((ret=alloc_subdevices(dev, 4))<0)
 		return ret;
 	if((ret=alloc_private(dev,sizeof(atmio16d_private)))<0)
 		return ret;
@@ -779,7 +756,7 @@ static int atmio16d_attach(comedi_device * dev, comedi_devconfig * it)
 	dev->read_subdev = s;
 	/* ai subdevice */
 	s->type=COMEDI_SUBD_AI;
-	s->subdev_flags=SDF_READABLE;
+	s->subdev_flags=SDF_READABLE|SDF_GROUND;
 	s->n_chan=(devpriv->adc_mux? 16 : 8);
 	s->len_chanlist=16;
 	s->insn_read = atmio16d_ai_insn_read;
@@ -802,7 +779,7 @@ static int atmio16d_attach(comedi_device * dev, comedi_devconfig * it)
 	/* ao subdevice */
 	s++;
 	s->type=COMEDI_SUBD_AO;
-	s->subdev_flags=SDF_WRITEABLE;
+	s->subdev_flags=SDF_WRITABLE;
 	s->n_chan=2;
 	s->insn_read=atmio16d_ao_insn_read;
 	s->insn_write=atmio16d_ao_insn_write;
@@ -829,7 +806,7 @@ static int atmio16d_attach(comedi_device * dev, comedi_devconfig * it)
 	/* Digital I/O */
 	s++;
 	s->type=COMEDI_SUBD_DIO;
-	s->subdev_flags=SDF_WRITEABLE|SDF_READABLE;
+	s->subdev_flags=SDF_WRITABLE|SDF_READABLE;
 	s->n_chan=8;
 	s->insn_bits = atmio16d_dio_insn_bits;
 	s->insn_config = atmio16d_dio_insn_config;
