@@ -29,7 +29,7 @@ Author: ds
 Status: works
 Devices: [National Instruments] PCI-DIO-32HS (ni_pcidio), PXI-6533,
   PCI-DIO-96, PCI-DIO-96B, PXI-6508, PCI-6503, PCI-6503B, PCI-6503X,
-  PXI-6503, PCI-6534, PCI-6533
+  PXI-6503, PCI-6533, PCI-6534
 Updated: Sun, 21 Apr 2002 21:03:38 -0700
 
 The DIO-96 appears as four 8255 subdevices.  See the 8255
@@ -44,6 +44,11 @@ DMA mostly works for the PCI-DIO32HS, but only in timed input mode.
 
 This driver could be easily modified to support AT-MIO32HS and
 AT-MIO96.
+
+The PCI-6534 requires a firmware upload after power-up to work, the
+firmware data and instructions for loading it with comedi_config
+it are contained in the
+comedi_nonfree_firmware tarball available from http://www.comedi.org
 */
 
 /*
@@ -62,7 +67,7 @@ AT-MIO96.
 
 #define USE_DMA
 //#define DEBUG 1
-#define DEBUG_FLAGS
+//#define DEBUG_FLAGS
 
 #include <linux/comedidev.h>
 
@@ -167,6 +172,7 @@ AT-MIO96.
 #define ClockReg			Protocol_Register_2
   #define ClockLine(x)		(((x)&3)<<5)
   #define InvertStopTrig	(1<<7)
+  #define DataLatching(x)       (((x)&3)<<5)
 
 #define Protocol_Register_3		67
 #define Sequence			Protocol_Register_3
@@ -240,6 +246,35 @@ AT-MIO96.
 #define Protocol_Register_8		88 /* 32 bit */
 #define StartDelay			Protocol_Register_8
 
+enum pci_6534_firmware_registers	/* 16 bit */
+{
+	Firmware_Control_Register = 0x100,
+	Firmware_Status_Register = 0x104,
+	Firmware_Data_Register = 0x108,
+	Firmware_Mask_Register = 0x10c,
+	Firmware_Debug_Register = 0x110,
+};
+/* main fpga registers (32 bit)*/
+enum pci_6534_fpga_registers
+{
+	FPGA_Control1_Register = 0x200,
+	FPGA_Control2_Register = 0x204,
+	FPGA_Irq_Mask_Register = 0x208,
+	FPGA_Status_Register = 0x20c,
+	FPGA_Signature_Register = 0x210,
+	FPGA_SCALS_Counter_Register = 0x280,	/*write-clear*/
+	FPGA_SCAMS_Counter_Register = 0x284,	/*write-clear*/
+	FPGA_SCBLS_Counter_Register = 0x288,	/*write-clear*/
+	FPGA_SCBMS_Counter_Register = 0x28c,	/*write-clear*/
+	FPGA_Temp_Control_Register = 0x2a0,
+	FPGA_DAR_Register = 0x2a8,
+	FPGA_ELC_Read_Register = 0x2b8,
+	FPGA_ELC_Write_Register = 0x2bc,
+};
+enum FPGA_Control_Bits
+{
+	FPGA_Enable_Bit = 0x8000,
+};
 
 #define TIMER_BASE 50		/* nanoseconds */
 
@@ -269,6 +304,7 @@ typedef struct{
 	char *name;
 	int n_8255;
 	unsigned int is_diodaq : 1;
+	unsigned int uses_firmware : 1;
 }nidio_board;
 static nidio_board nidio_boards[]={
 	{
@@ -288,6 +324,7 @@ static nidio_board nidio_boards[]={
 	name:		"pci-6534",
 	n_8255:		0,
 	is_diodaq:	1,
+	uses_firmware:	1,
 	},
 	{
 	dev_id:		0x0160,
@@ -354,6 +391,7 @@ typedef struct{
 	struct mite_struct *mite;
 	int boardtype;
 	int dio;
+	unsigned short OpModeBits;
 }nidio96_private;
 #define devpriv ((nidio96_private *)dev->private)
 
@@ -403,7 +441,7 @@ static irqreturn_t nidio_interrupt(int irq, void *d, struct pt_regs *regs)
 
 	status = readb(dev->iobase+Interrupt_And_Window_Status);
 	flags = readb(dev->iobase+Group_1_Flags);
-	m_status = readl(mite->mite_io_addr + MITE_CHSR + CHAN_OFFSET( DI_DMA_CHAN ));
+	m_status = readl(mite->mite_io_addr + MITE_CHSR(DI_DMA_CHAN));
 
 	//interrupcions parasites
 	if(dev->attached == 0){
@@ -428,7 +466,7 @@ static irqreturn_t nidio_interrupt(int irq, void *d, struct pt_regs *regs)
 		if(m_status & CHSR_LINKC){
 			unsigned int count;
 
-			writel(CHOR_CLRLC, mite->mite_io_addr + MITE_CHOR + CHAN_OFFSET(DI_DMA_CHAN));
+			writel(CHOR_CLRLC, mite->mite_io_addr + MITE_CHOR(DI_DMA_CHAN));
 			count = le32_to_cpu(mite_chan->ring[mite_chan->current_link].count);
 
 			/* XXX need to byteswap */
@@ -444,13 +482,11 @@ static irqreturn_t nidio_interrupt(int irq, void *d, struct pt_regs *regs)
 			}
 		}
 		if(m_status & CHSR_DONE){
-			writel(CHOR_CLRDONE, mite->mite_io_addr + MITE_CHOR +
-				CHAN_OFFSET( DI_DMA_CHAN ));
+			writel(CHOR_CLRDONE, mite->mite_io_addr + MITE_CHOR(DI_DMA_CHAN));
 		}
-		if(m_status & ~(CHSR_INT | CHSR_LINKC | CHSR_DONE | CHSR_DRDY | CHSR_DRQ1)){
+		if(m_status & ~(CHSR_INT | CHSR_LINKC | CHSR_DONE | CHSR_DRDY | CHSR_DRQ1 | CHSR_MRDY)){
 			DPRINTK("unknown mite interrupt, disabling IRQ\n");
-			writel(CHOR_DMARESET, mite->mite_io_addr + MITE_CHOR +
-				CHAN_OFFSET( DI_DMA_CHAN ));
+			writel(CHOR_DMARESET, mite->mite_io_addr + MITE_CHOR(DI_DMA_CHAN));
 			disable_irq(dev->irq);
 		}
 		async->events |= COMEDI_CB_BLOCK;
@@ -499,8 +535,7 @@ static irqreturn_t nidio_interrupt(int irq, void *d, struct pt_regs *regs)
 			writeb(0x00,dev->iobase+Master_DMA_And_Interrupt_Control);
 #ifdef USE_DMA
 			mite_dma_disarm(mite, DI_DMA_CHAN);
-			writel(CHOR_DMARESET, mite->mite_io_addr + MITE_CHOR +
-				CHAN_OFFSET( DI_DMA_CHAN ));
+			writel(CHOR_DMARESET, mite->mite_io_addr + MITE_CHOR(DI_DMA_CHAN));
 #endif
 			break;
 		}else if(flags & Waited){
@@ -510,8 +545,7 @@ static irqreturn_t nidio_interrupt(int irq, void *d, struct pt_regs *regs)
 			async->events |= COMEDI_CB_EOA | COMEDI_CB_ERROR;
 #ifdef USE_DMA
 			mite_dma_disarm(mite, DI_DMA_CHAN);
-			writel(CHOR_DMARESET, mite->mite_io_addr + MITE_CHOR +
-				CHAN_OFFSET( DI_DMA_CHAN ));
+			writel(CHOR_DMARESET, mite->mite_io_addr + MITE_CHOR(DI_DMA_CHAN));
 #endif
 			break;
 		}else if(flags & PrimaryTC){
@@ -621,12 +655,16 @@ static int ni_pcidio_insn_config(comedi_device *dev,comedi_subdevice *s,
 {
 	if(insn->n!=1)return -EINVAL;
 	switch(data[0]){
-	case COMEDI_OUTPUT:
+	case INSN_CONFIG_DIO_OUTPUT:
 		s->io_bits |= 1<<CR_CHAN(insn->chanspec);
 		break;
-	case COMEDI_INPUT:
+	case INSN_CONFIG_DIO_INPUT:
 		s->io_bits &= ~(1<<CR_CHAN(insn->chanspec));
 		break;
+	case INSN_CONFIG_DIO_QUERY:
+		data[1] = (s->io_bits & (1<<CR_CHAN(insn->chanspec))) ? COMEDI_OUTPUT : COMEDI_INPUT;
+		return insn->n;
+		break;
 	default:
 		return -EINVAL;
 	}
@@ -670,11 +708,11 @@ static int ni_pcidio_cmdtest(comedi_device *dev,comedi_subdevice *s,
 	if(!cmd->convert_src || tmp!=cmd->convert_src)err++;
 
 	tmp=cmd->scan_end_src;
-	cmd->scan_end_src &= TRIG_COUNT;
+	cmd->scan_end_src &= TRIG_COUNT;  
 	if(!cmd->scan_end_src || tmp!=cmd->scan_end_src)err++;
 
 	tmp=cmd->stop_src;
-	cmd->stop_src &= TRIG_COUNT;
+	cmd->stop_src &= TRIG_COUNT|TRIG_NONE;
 	if(!cmd->stop_src || tmp!=cmd->stop_src)err++;
 
 	if(err)return 1;
@@ -780,7 +818,7 @@ static int ni_pcidio_cmd(comedi_device *dev,comedi_subdevice *s)
 	/* XXX configure ports for input*/
 	writel(0x0000,dev->iobase+Port_Pin_Directions(0));
 
-	if(0){
+	if(1){
 		/* enable fifos A B C D */
 		writeb(0x0f,dev->iobase+Data_Path);
 
@@ -856,9 +894,14 @@ static int ni_pcidio_cmd(comedi_device *dev,comedi_subdevice *s)
 	writeb(IntEn,dev->iobase+Interrupt_Control);
 	writeb(0x03,dev->iobase+Master_DMA_And_Interrupt_Control);
 
+	if(cmd->stop_src == TRIG_NONE) {
+		devpriv->OpModeBits = DataLatching(0) | RunMode(7);
+	} else {  // TRIG_TIMER
+		devpriv->OpModeBits = Numbered | RunMode(7);
+	}
 	if(cmd->start_src == TRIG_NOW){
 		/* start */
-		writeb(Numbered | RunMode(7),dev->iobase+OpMode);
+		writeb(devpriv->OpModeBits, dev->iobase+OpMode);
 		s->async->inttrig = NULL;
 	}else{
 		/* TRIG_INT */
@@ -879,7 +922,7 @@ static void setup_mite_dma(comedi_device *dev,comedi_subdevice *s)
 
 	mite_chan->dir = COMEDI_INPUT;
 
-	mite_prep_dma(mite, DI_DMA_CHAN, 16, 16);
+	mite_prep_dma(mite, DI_DMA_CHAN, 32, 32);
 
 	mite_dma_arm(mite, DI_DMA_CHAN);
 }
@@ -890,7 +933,7 @@ static int ni_pcidio_inttrig(comedi_device *dev, comedi_subdevice *s,
 {
 	if(trignum!=0)return -EINVAL;
 
-	writeb(Numbered | RunMode(7),dev->iobase+OpMode);
+	writeb(devpriv->OpModeBits, dev->iobase+OpMode);
 	s->async->inttrig = NULL;
 
 	return 1;
@@ -918,6 +961,105 @@ static int ni_pcidio_change(comedi_device *dev, comedi_subdevice *s,
 	return 0;
 }
 
+static int pci_6534_load_fpga(comedi_device *dev, int fpga_index, u8 *data, int data_len)
+{
+	static const int timeout = 1000;
+	int i, j;
+	writew(0x80 | fpga_index, dev->iobase + Firmware_Control_Register);
+	writew(0xc0 | fpga_index, dev->iobase + Firmware_Control_Register);
+	for(i = 0; (readw(dev->iobase + Firmware_Status_Register) & 0x2) == 0 && i < timeout; ++i)
+	{
+		udelay(1);
+	}
+	if(i == timeout)
+	{
+		printk("ni_pcidio: failed to load fpga %i, waiting for status 0x2\n", fpga_index);
+		return -EIO;
+	}
+	writew(0x80 | fpga_index, dev->iobase + Firmware_Control_Register);
+	for(i = 0; readw(dev->iobase + Firmware_Status_Register) != 0x3 && i < timeout; ++i)
+	{
+		udelay(1);
+	}
+	if(i == timeout)
+	{
+		printk("ni_pcidio: failed to load fpga %i, waiting for status 0x3\n", fpga_index);
+		return -EIO;
+	}
+	for(j = 0; j + 1 < data_len;)
+	{
+		unsigned int value = data[j++];
+		value |= data[j++] << 8;
+		writew(value, dev->iobase + Firmware_Data_Register);
+		for(i = 0; (readw(dev->iobase + Firmware_Status_Register) & 0x2) == 0 && i < timeout; ++i)
+		{
+			udelay(1);
+		}
+		if(i == timeout)
+		{
+			printk("ni_pcidio: failed to load word into fpga %i\n", fpga_index);
+			return -EIO;
+		}
+		if(need_resched())
+			schedule();
+	}
+	writew(0x0, dev->iobase + Firmware_Control_Register);
+	return 0;
+}
+
+static int pci_6534_reset_fpga(comedi_device *dev, int fpga_index)
+{
+	return pci_6534_load_fpga(dev, fpga_index, NULL, 0);
+}
+
+static int pci_6534_reset_fpgas(comedi_device *dev)
+{
+	int ret;
+	int i;
+	writew(0x0, dev->iobase + Firmware_Control_Register);
+	for(i = 0; i < 3; ++i)
+	{
+		ret = pci_6534_reset_fpga(dev, i);
+		if(ret < 0) break;
+	}
+	writew(0x0, dev->iobase + Firmware_Mask_Register);
+	return ret;
+}
+
+static void pci_6534_init_main_fpga(comedi_device *dev)	
+{
+	writel(0, dev->iobase + FPGA_Control1_Register);
+	writel(0, dev->iobase + FPGA_Control2_Register);
+	writel(0, dev->iobase + FPGA_SCALS_Counter_Register);
+	writel(0, dev->iobase + FPGA_SCAMS_Counter_Register);
+	writel(0, dev->iobase + FPGA_SCBLS_Counter_Register);
+	writel(0, dev->iobase + FPGA_SCBMS_Counter_Register);
+}
+
+static int pci_6534_upload_firmware(comedi_device *dev, int options[])
+{
+	int ret;
+	void *main_fpga_data, *scarab_a_data, *scarab_b_data;
+	int main_fpga_data_len, scarab_a_data_len, scarab_b_data_len;
+	
+	if(options[COMEDI_DEVCONF_AUX_DATA_LENGTH] == 0) return 0;
+	ret = pci_6534_reset_fpgas(dev);
+	if(ret < 0) return ret;
+	main_fpga_data = comedi_aux_data(options, 0);
+	main_fpga_data_len = options[COMEDI_DEVCONF_AUX_DATA0_LENGTH];
+	ret = pci_6534_load_fpga(dev, 2, main_fpga_data, main_fpga_data_len);
+	if(ret < 0) return ret;
+	pci_6534_init_main_fpga(dev);
+	scarab_a_data = comedi_aux_data(options, 1);
+	scarab_a_data_len = options[COMEDI_DEVCONF_AUX_DATA1_LENGTH];	
+	ret = pci_6534_load_fpga(dev, 0, scarab_a_data, scarab_a_data_len);
+	if(ret < 0) return ret;
+	scarab_b_data = comedi_aux_data(options, 2);
+	scarab_b_data_len = options[COMEDI_DEVCONF_AUX_DATA2_LENGTH];	
+	ret = pci_6534_load_fpga(dev, 1, scarab_b_data, scarab_b_data_len);
+	if(ret < 0) return ret;	
+	return 0;
+}
 
 static int nidio_attach(comedi_device *dev,comedi_devconfig *it)
 {
@@ -945,7 +1087,12 @@ static int nidio_attach(comedi_device *dev,comedi_devconfig *it)
 	dev->board_name=this_board->name;
 	dev->irq=mite_irq(devpriv->mite);
 	printk(" %s",dev->board_name);
-
+	if(this_board->uses_firmware)
+	{
+		ret = pci_6534_upload_firmware(dev, it->options);
+		if(ret < 0)
+			return ret;
+	}
 	if(!this_board->is_diodaq){
 		n_subdevices=this_board->n_8255;
 	}else{
@@ -967,7 +1114,7 @@ static int nidio_attach(comedi_device *dev,comedi_devconfig *it)
 
 		dev->read_subdev = s;
 		s->type=COMEDI_SUBD_DIO;
-		s->subdev_flags=SDF_READABLE|SDF_WRITABLE;
+		s->subdev_flags=SDF_READABLE|SDF_WRITABLE|SDF_LSAMPL;
 		s->n_chan=32;
 		s->range_table=&range_digital;
 		s->maxdata=1;
