@@ -26,7 +26,7 @@ Author: ds, John Hallen, Frank Mori Hess, Rolf Mueller, Herbert Peremans,
   Herman Bruyninckx
 Status: works
 Devices: [National Instruments] PCI-MIO-16XE-50 (ni_pcimio),
-  PCI-MIO-16XE-10, PXI-6030E, PCI-MIO-16E-1, PCI-MIO-16E-4, PCI-6040E,
+  PCI-MIO-16XE-10, PXI-6030E, PCI-MIO-16E-1, PCI-MIO-16E-4, PCI-6014, PCI-6040E,
   PXI-6040E, PCI-6031E, PCI-6032E, PCI-6033E, PCI-6071E, PCI-6023E,
   PCI-6024E, PCI-6025E, PXI-6025E, PCI-6034E, PCI-6035E, PCI-6052E,
   PCI-6110, PCI-6111, PCI-6711, PXI-6711, PCI-6713, PXI-6713,
@@ -136,6 +136,7 @@ static struct pci_device_id ni_pci_table[] __devinitdata = {
 	{ PCI_VENDOR_ID_NATINST, 0x18c0, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
 	{ PCI_VENDOR_ID_NATINST, 0x1580, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
 	{ PCI_VENDOR_ID_NATINST, 0x2890, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
+	{ PCI_VENDOR_ID_NATINST, 0x28c0, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0 },
 	{ 0 }
 };
 MODULE_DEVICE_TABLE(pci, ni_pci_table);
@@ -171,6 +172,21 @@ static ni_board ni_boards[]={
 		caldac:         {dac8800,dac8043,ad8522},
 		has_8255:       0,
 	},
+	{	device_id:      0x28c0,
+		name:           "pci-6014",
+		n_adchan:       16,
+		adbits:         16,
+		ai_fifo_depth:  512,
+		alwaysdither:   1,
+		gainlkup:       ai_gain_4,
+		ai_speed:       5000,
+		n_aochan:       2,
+		aobits:         16,
+		ao_fifo_depth:  0,
+		ao_unipolar:    0,
+		caldac:         {ad8804_debug},
+		has_8255:       0,
+	},
 	{       device_id:      0x11d0,
 		name:           "pxi-6030e",
 		n_adchan:       16,
@@ -650,6 +666,14 @@ static comedi_driver driver_pcimio={
 };
 COMEDI_INITCLEANUP(driver_pcimio);
 
+typedef struct{
+	struct mite_struct *mite;
+
+	NI_PRIVATE_COMMON
+
+	unsigned int last_buf_write_count;
+}ni_private;
+#define devpriv ((ni_private *)dev->private)
 
 /* How we access registers */
 
@@ -675,10 +699,10 @@ static inline void __win_out(comedi_device *dev, unsigned short data, int addr)
 {
 	unsigned long flags;
 
-	comedi_spin_lock_irqsave(&dev->spinlock,flags);
+	comedi_spin_lock_irqsave(&devpriv->window_lock,flags);
 	ni_writew(addr,Window_Address);
 	ni_writew(data,Window_Data);
-	comedi_spin_unlock_irqrestore(&dev->spinlock,flags);
+	comedi_spin_unlock_irqrestore(&devpriv->window_lock,flags);
 }
 
 #define win_in(addr) __win_in(dev,addr)
@@ -687,10 +711,10 @@ static inline unsigned short __win_in(comedi_device *dev, int addr)
 	unsigned long flags;
 	int ret;
 
-	comedi_spin_lock_irqsave(&dev->spinlock,flags);
+	comedi_spin_lock_irqsave(&devpriv->window_lock,flags);
 	ni_writew(addr,Window_Address);
 	ret = ni_readw(Window_Data);
-	comedi_spin_unlock_irqrestore(&dev->spinlock,flags);
+	comedi_spin_unlock_irqrestore(&devpriv->window_lock,flags);
 
 	return ret;
 }
@@ -700,17 +724,6 @@ static inline unsigned short __win_in(comedi_device *dev, int addr)
 
 #define NI_E_IRQ_FLAGS		SA_SHIRQ
 
-
-typedef struct{
-	struct mite_struct *mite;
-
-	NI_PRIVATE_COMMON
-
-	unsigned int last_buf_write_count;
-}ni_private;
-#define devpriv ((ni_private *)dev->private)
-
-
 #include "ni_mio_common.c"
 
 
@@ -742,7 +755,7 @@ static int pcimio_attach(comedi_device *dev,comedi_devconfig *it)
 
 	printk("comedi%d: ni_pcimio:",dev->minor);
 
-	ret=alloc_private(dev,sizeof(ni_private));
+	ret=ni_alloc_private(dev);
 	if(ret<0)return ret;
 
 	ret=pcimio_find_device(dev,it->options[0],it->options[1]);
