@@ -438,7 +438,7 @@ static int pcl812_ai_insn_read(comedi_device *dev,comedi_subdevice *s,
 				goto conv_finish;
 			comedi_udelay(1);
 		}
-		rt_printk("comedi%d: pcl812: (%s at 0x%x) A/D insn read timeout\n", dev->minor, dev->board_name, dev->iobase);
+		rt_printk("comedi%d: pcl812: (%s at 0x%lx) A/D insn read timeout\n", dev->minor, dev->board_name, dev->iobase);
 		outb(devpriv->mode_reg_int|0, dev->iobase + PCL812_MODE);
 		return -ETIME;
 
@@ -469,7 +469,7 @@ static int acl8216_ai_insn_read(comedi_device *dev,comedi_subdevice *s,
 				goto conv_finish;
 			comedi_udelay(1);
 		}
-		rt_printk("comedi%d: pcl812: (%s at 0x%x) A/D insn read timeout\n", dev->minor, dev->board_name, dev->iobase);
+		rt_printk("comedi%d: pcl812: (%s at 0x%lx) A/D insn read timeout\n", dev->minor, dev->board_name, dev->iobase);
 		outb(0, dev->iobase + PCL812_MODE);
 		return -ETIME;
 
@@ -877,7 +877,7 @@ static irqreturn_t interrupt_pcl812_ai_int(int irq, void *d, struct pt_regs *reg
 	}
 
 	if (err) {
-		rt_printk("comedi%d: pcl812: (%s at 0x%x) A/D cmd IRQ without DRDY!\n", dev->minor, dev->board_name, dev->iobase);
+		rt_printk("comedi%d: pcl812: (%s at 0x%lx) A/D cmd IRQ without DRDY!\n", dev->minor, dev->board_name, dev->iobase);
 		pcl812_ai_cancel(dev,s);
 		s->async->events |= COMEDI_CB_EOA|COMEDI_CB_ERROR;
 		comedi_event(dev,s,s->async->events);
