@@ -774,7 +774,8 @@ static int rtd_attach (
     /*
      * Probe the device to determine what device in the series it is.
      */
-    pci_for_each_dev (pcidev) {
+	for(pcidev = pci_get_device(PCI_ANY_ID, PCI_ANY_ID, NULL); pcidev != NULL ; 
+		pcidev = pci_get_device(PCI_ANY_ID, PCI_ANY_ID, pcidev)) {
 	if (pcidev->vendor == PCI_VENDOR_ID_RTD) {
 	    if (it->options[0] || it->options[1]) {
 		if (pcidev->bus->number == it->options[0]
@@ -799,12 +800,12 @@ static int rtd_attach (
 	return -EIO;
     }
 
+    devpriv->pci_dev = pcidev;
     if (pcidev->device != thisboard->device_id) {
 	printk ("Found an RTD card, but not the supported type (%x).\n",
 		pcidev->device);
 	return -EIO;
     }
-    devpriv->pci_dev = pcidev;
     dev->board_name = thisboard->name;
 
     if((ret=pci_enable_device(pcidev))<0){
@@ -1021,6 +1022,7 @@ static int rtd_attach (
 
     return 1;
 
+#if 0
     /* hit an error, clean up memory and return ret */
 //rtd_attach_die_error:
 #ifdef USE_DMA
@@ -1062,7 +1064,11 @@ static int rtd_attach (
     if (devpriv->lcfg) {
 	iounmap (devpriv->lcfg);
     }
+    if (devpriv->pci_dev) {
+	pci_dev_put(devpriv->pci_dev);
+    }
     return ret;
+#endif
 }
 
 /*
@@ -1138,6 +1144,9 @@ static int rtd_detach (
 	if (devpriv->lcfg) {
 	    iounmap (devpriv->lcfg);
 	}
+	if (devpriv->pci_dev) {
+	    pci_dev_put(devpriv->pci_dev);
+	}
     }
 
     printk("comedi%d: rtd520: removed.\n",dev->minor);
@@ -2257,18 +2266,25 @@ static int rtd_dio_insn_config (
 {
     int chan=CR_CHAN(insn->chanspec);
 
-    if (insn->n!=1) return -EINVAL;
-
     /* The input or output configuration of each digital line is
      * configured by a special insn_config instruction.  chanspec
      * contains the channel to be changed, and data[0] contains the 
      * value COMEDI_INPUT or COMEDI_OUTPUT. */
-	
-    if (data[0]==COMEDI_OUTPUT) {
-	s->io_bits |= 1<<chan;		/* 1 means Out */
-    } else {
-	s->io_bits &= ~(1<<chan);
-    }
+	switch(data[0])
+	{
+	case INSN_CONFIG_DIO_OUTPUT:
+		s->io_bits |= 1<<chan;		/* 1 means Out */
+		break;
+	case INSN_CONFIG_DIO_INPUT:
+		s->io_bits &= ~(1<<chan);
+		break;
+	case INSN_CONFIG_DIO_QUERY:
+		data[1] = (s->io_bits & (1<<chan)) ? COMEDI_OUTPUT : COMEDI_INPUT;
+		return insn->n;
+		break;
+	default:
+		return -EINVAL;
+	}
 
     DPRINTK("rtd520: port_0_direction=0x%x (1 means out)\n", s->io_bits);
     /* TODO support digital match interrupts and strobes */
