@@ -561,21 +561,29 @@ static int skel_dio_insn_config(comedi_device *dev,comedi_subdevice *s,
 {
 	int chan=CR_CHAN(insn->chanspec);
 
-	if(insn->n!=1)return -EINVAL;
-
 	/* The input or output configuration of each digital line is
 	 * configured by a special insn_config instruction.  chanspec
 	 * contains the channel to be changed, and data[0] contains the
 	 * value COMEDI_INPUT or COMEDI_OUTPUT. */
-
-	if(data[0]==COMEDI_OUTPUT){
+	switch(data[0])
+	{
+	case INSN_CONFIG_DIO_OUTPUT:
 		s->io_bits |= 1<<chan;
-	}else{
+		break;
+	case INSN_CONFIG_DIO_INPUT:
 		s->io_bits &= ~(1<<chan);
+		break;
+	case INSN_CONFIG_DIO_QUERY:
+		data[1] = (s->io_bits & (1 << chan)) ? COMEDI_OUTPUT : COMEDI_INPUT;
+		return insn->n;
+		break;
+	default:
+		return -EINVAL;
+		break;
 	}
 	//outw(s->io_bits,dev->iobase + SKEL_DIO_CONFIG);
 
-	return 1;
+	return insn->n;
 }
 
 /*
