@@ -76,8 +76,8 @@ struct comedi_irq_struct {
 	comedi_device *dev_id;
 };
 
-static int rt_get_irq(struct comedi_irq_struct *it);
-static int rt_release_irq(struct comedi_irq_struct *it);
+static int comedi_rt_get_irq(struct comedi_irq_struct *it);
+static int comedi_rt_release_irq(struct comedi_irq_struct *it);
 
 #define MAX_IRQ_SHARING 6
 static struct comedi_irq_struct *comedi_irqs[NR_IRQS][MAX_IRQ_SHARING];
@@ -129,16 +129,16 @@ static int insert_comedi_irq_struct( int irq,
 	return -1;
 }
 
-int comedi_request_irq(unsigned irq,void (*handler)(int, void *,struct pt_regs *),
+int comedi_request_irq(unsigned irq, irqreturn_t (*handler)(int, void *,struct pt_regs *),
 		unsigned long flags,const char *device,comedi_device *dev_id)
 {
 	struct comedi_irq_struct *it;
 	int ret;
 
-	it=kmalloc(sizeof(*it),GFP_KERNEL);
+	it=kmalloc(sizeof(struct comedi_irq_struct),GFP_KERNEL);
 	if(!it)
 		return -ENOMEM;
-	memset(it,0,sizeof(*it));
+	memset(it,0,sizeof(struct comedi_irq_struct));
 
 	it->handler=handler;
 	it->irq=irq;
@@ -182,7 +182,7 @@ void comedi_free_irq(unsigned int irq,comedi_device *dev_id)
 
 	if(it->rt){
 		printk("real-time IRQ allocated at board removal (ignore)\n");
-		rt_release_irq(it);
+		comedi_rt_release_irq(it);
 	}
 
 	free_irq(it->irq,it->dev_id);
@@ -203,14 +203,14 @@ int comedi_switch_to_rt(comedi_device *dev)
 
 	/* rt interrupts and shared interrupts don't mix */
 	if(it->flags & SA_SHIRQ){
-		DPRINTK( "cannot switch shared interrupt to real time priority\n" );
+		rt_printk("comedi: cannot switch shared interrupt to RT priority\n");
 		return -1;
 	}
 
 	comedi_spin_lock_irqsave( &dev->spinlock, flags );
 
 	if(!dev->rt)
-		rt_get_irq(it);
+		comedi_rt_get_irq(it);
 
 	dev->rt++;
 	it->rt=1;
@@ -237,7 +237,7 @@ void comedi_switch_to_non_rt(comedi_device *dev)
 
 	dev->rt--;
 	if(!dev->rt)
-		rt_release_irq(it);
+		comedi_rt_release_irq(it);
 
 	it->rt=0;
 
@@ -258,9 +258,6 @@ void comedi_rt_pend_wakeup(wait_queue_head_t *q)
 /* RTAI section */
 #ifdef CONFIG_COMEDI_RTAI
 
-#ifdef CONFIG_PPC
-#define HAVE_RT_REQUEST_IRQ_WITH_ARG
-#endif
 #ifndef HAVE_RT_REQUEST_IRQ_WITH_ARG
 #define DECLARE_VOID_IRQ(irq) \
 static void handle_void_irq_ ## irq (void){ handle_void_irq(irq);}
@@ -276,6 +273,7 @@ static void handle_void_irq(int irq)
 		if( it == NULL ) continue;
 		it->handler( irq, it->dev_id, NULL );
 	}
+	rt_enable_irq(irq);	//needed by rtai-adeos, seems like it shouldn't hurt earlier versions
 }
 
 DECLARE_VOID_IRQ(0);
@@ -332,7 +330,7 @@ static V_FP_V handle_void_irq_ptrs[]={
 };
 /* if you need more, fix it yourself... */
 
-static int rt_get_irq(struct comedi_irq_struct *it)
+static int comedi_rt_get_irq(struct comedi_irq_struct *it)
 {
 	rt_request_global_irq(it->irq,handle_void_irq_ptrs[it->irq]);
 	rt_startup_irq(it->irq);
@@ -340,7 +338,7 @@ static int rt_get_irq(struct comedi_irq_struct *it)
 	return 0;
 }
 
-static int rt_release_irq(struct comedi_irq_struct *it)
+static int comedi_rt_release_irq(struct comedi_irq_struct *it)
 {
 	rt_shutdown_irq(it->irq);
 	rt_free_global_irq(it->irq);
@@ -348,7 +346,7 @@ static int rt_release_irq(struct comedi_irq_struct *it)
 }
 #else
 
-static int rt_get_irq(struct comedi_irq_struct *it)
+static int comedi_rt_get_irq(struct comedi_irq_struct *it)
 {
 	int ret;
 
@@ -363,7 +361,7 @@ static int rt_get_irq(struct comedi_irq_struct *it)
 	return 0;
 }
 
-static int rt_release_irq(struct comedi_irq_struct *it)
+static int comedi_rt_release_irq(struct comedi_irq_struct *it)
 {
 	rt_shutdown_irq(it->irq);
 	rt_free_global_irq(it->irq);
@@ -405,13 +403,13 @@ static unsigned int handle_rtl_irq(unsigned int irq,struct pt_regs *regs)
 	return 0;
 }
 
-static int rt_get_irq(struct comedi_irq_struct *it)
+static int comedi_rt_get_irq(struct comedi_irq_struct *it)
 {
 	rtl_request_global_irq(it->irq,handle_rtl_irq);
 	return 0;
 }
 
-static int rt_release_irq(struct comedi_irq_struct *it)
+static int comedi_rt_release_irq(struct comedi_irq_struct *it)
 {
 	rtl_free_global_irq(it->irq);
 	return 0;
@@ -430,7 +428,7 @@ void comedi_rt_cleanup(void)
 #endif
 
 #ifdef CONFIG_COMEDI_PIRQ
-static int rt_get_irq(struct comedi_irq_struct *it)
+static int comedi_rt_get_irq(struct comedi_irq_struct *it)
 {
 	int ret;
 
@@ -441,7 +439,7 @@ static int rt_get_irq(struct comedi_irq_struct *it)
 	return ret;
 }
 
-static int rt_release_irq(struct comedi_irq_struct *it)
+static int comedi_rt_release_irq(struct comedi_irq_struct *it)
 {
 	int ret;
 
