@@ -6,7 +6,7 @@
     Copyright (C) 1998-2001 David A. Schleef <ds@schleef.org>
 
     This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
+    it under the terms of the GNU Lesser General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.
 
@@ -37,12 +37,18 @@ extern "C" {
    kernel structures are currently statically allocated, thus you
    don't want this to be much more than you actually use.
  */
-#define COMEDI_NDEVICES 4
+#define COMEDI_NDEVICES 16
 
 /* number of config options in the config structure */
 #define COMEDI_NDEVCONFOPTS 32
-#define COMEDI_DEVCONF_AUX_DATA		30
-#define COMEDI_DEVCONF_AUX_DATA_LENGTH	31
+/*length of nth chunk of firmware data*/
+#define COMEDI_DEVCONF_AUX_DATA3_LENGTH		25
+#define COMEDI_DEVCONF_AUX_DATA2_LENGTH		26
+#define COMEDI_DEVCONF_AUX_DATA1_LENGTH		27
+#define COMEDI_DEVCONF_AUX_DATA0_LENGTH		28
+#define COMEDI_DEVCONF_AUX_DATA_HI		29	/*most significant 32 bits of pointer address (if needed)*/
+#define COMEDI_DEVCONF_AUX_DATA_LO		30	/*least significant 32 bits of pointer address*/
+#define COMEDI_DEVCONF_AUX_DATA_LENGTH	31	/* total data length */
 
 /* max length of device and driver names */
 #define COMEDI_NAMELEN 20
@@ -95,7 +101,6 @@ typedef unsigned short sampl_t;
 #define GPCT_CONT_PULSE_OUT	0x0200
 #define GPCT_SINGLE_PULSE_OUT	0x0400
 
-
 /* instructions */
 
 #define INSN_MASK_WRITE		0x8000000
@@ -118,7 +123,7 @@ typedef unsigned short sampl_t;
 #define TRIG_DEGLITCH	0x0004		/* enable deglitching */
 //#define TRIG_RT	0x0008		/* perform op in real time */
 #define TRIG_CONFIG	0x0010		/* perform configuration, not triggering */
-//#define TRIG_WAKE_EOS	0x0020		/* wake up on end-of-scan events */
+#define TRIG_WAKE_EOS	0x0020		/* wake up on end-of-scan events */
 //#define TRIG_WRITE	0x0040		/* write to bidirectional devices */
 
 /* command flags */
@@ -127,7 +132,6 @@ typedef unsigned short sampl_t;
 #define CMDF_PRIORITY		0x00000008 /* try to use a real-time interrupt while performing command */
 
 #define TRIG_RT		CMDF_PRIORITY /* compatibility definition */
-#define TRIG_WAKE_EOS		0x00000020 /* legacy definition for COMEDI_EV_SCAN_END */
 
 #define CMDF_WRITE		0x00000040
 #define TRIG_WRITE	CMDF_WRITE /* compatibility definition */
@@ -206,23 +210,39 @@ typedef unsigned short sampl_t;
 #define COMEDI_SUBD_MEMORY              8	/* memory, EEPROM, DPRAM */
 #define COMEDI_SUBD_CALIB               9	/* calibration DACs */
 #define COMEDI_SUBD_PROC                10	/* processor, DSP */
+#define COMEDI_SUBD_SERIAL              11	/* serial IO */
 
 /* configuration instructions */
 
-#define COMEDI_INPUT			0
-#define COMEDI_OUTPUT			1
-#define COMEDI_OPENDRAIN		2
+enum configuration_ids
+{
+	INSN_CONFIG_DIO_INPUT = 0,
+	INSN_CONFIG_DIO_OUTPUT = 1,
+	INSN_CONFIG_DIO_OPENDRAIN = 2,
+	COMEDI_INPUT = INSN_CONFIG_DIO_INPUT,
+	COMEDI_OUTPUT = INSN_CONFIG_DIO_OUTPUT,
+	COMEDI_OPENDRAIN = INSN_CONFIG_DIO_OPENDRAIN,
+	INSN_CONFIG_ANALOG_TRIG = 16,
+//	INSN_CONFIG_WAVEFORM = 17,
+//	INSN_CONFIG_TRIG = 18,
+//	INSN_CONFIG_COUNTER = 19,
+	INSN_CONFIG_ALT_SOURCE = 20,
+	INSN_CONFIG_DIGITAL_TRIG = 21,
+	INSN_CONFIG_BLOCK_SIZE = 22,
+	INSN_CONFIG_TIMER_1 = 23,
+	INSN_CONFIG_FILTER = 24,
+	INSN_CONFIG_CHANGE_NOTIFY = 25,
 
-#define INSN_CONFIG_ANALOG_TRIG		0x10
-//#define INSN_CONFIG_WAVEFORM		0x11
-//#define INSN_CONFIG_TRIG		0x12
-//#define INSN_CONFIG_COUNTER		0x13
-#define INSN_CONFIG_ALT_SOURCE		0x14
-#define INSN_CONFIG_DIGITAL_TRIG	0x15
-#define INSN_CONFIG_BLOCK_SIZE		0x16
-#define INSN_CONFIG_TIMER_1		0x17
-#define INSN_CONFIG_FILTER		0x18
-#define INSN_CONFIG_CHANGE_NOTIFY	0x19
+	/*ALPHA*/
+	INSN_CONFIG_SERIAL_CLOCK = 26,
+	INSN_CONFIG_BIDIRECTIONAL_DATA = 27,
+	INSN_CONFIG_DIO_QUERY = 28,
+	INSN_CONFIG_GPCT_SINGLE_PULSE_GENERATOR = 1001, // Use CTR as single pulsegenerator
+	INSN_CONFIG_GPCT_PULSE_TRAIN_GENERATOR = 1002, // Use CTR as pulsetraingenerator
+	INSN_CONFIG_GPCT_QUADRATURE_ENCODER = 1003, // Use the counter as encoder
+	INSN_CONFIG_8254_SET_MODE = 4097,
+	INSN_CONFIG_8254_READ_STATUS = 4098
+};
 
 /* ioctls */
 
@@ -376,10 +396,10 @@ struct comedi_bufinfo_struct{
 	unsigned int subdevice;
 	unsigned int bytes_read;
 
-	unsigned int buf_int_ptr;
-	unsigned int buf_user_ptr;
-	unsigned int buf_int_count;
-	unsigned int buf_user_count;
+	unsigned int buf_write_ptr;
+	unsigned int buf_read_ptr;
+	unsigned int buf_write_count;
+	unsigned int buf_read_count;
 
 	unsigned int bytes_written;
 
@@ -412,7 +432,62 @@ struct comedi_bufinfo_struct{
 #define COMEDI_CB_ERROR		16	/* card error during acquisition */
 #define COMEDI_CB_OVERFLOW	32	/* buffer overflow/underflow */
 
+/**********************************************************/
+/* everything after this line is ALPHA */
+/**********************************************************/
 
+/*
+	Added by Klaas Gadeyne after implementation of driver for comedi NI
+	660x counter card.
+	Please see
+	<http://people.mech.kuleuven.ac.be/~kgadeyne/linux/> for more
+	information about their use
+*/
+
+// X1 encoding
+#define GPCT_X1                 0x01
+// X2 encoding
+#define GPCT_X2                 0x02
+// X3 encoding
+#define GPCT_X4                 0x04 
+// When to take into account the indexpulse:
+#define GPCT_IndexPhaseHighHigh 0
+#define GPCT_IndexPhaseLowHigh 1
+#define GPCT_IndexPhaseLowLow 2
+#define GPCT_IndexPhaseHighLow 3
+// Reset when index pulse arrives?
+#define GPCT_RESET_COUNTER_ON_INDEX 1
+
+/*
+  8254 specific configuration.
+
+  It supports two config commands:
+
+  0 ID: INSN_CONFIG_8254_SET_MODE
+  1 8254 Mode
+    I8254_MODE0, I8254_MODE1, ..., I8254_MODE5
+    OR'ed with:
+    I8254_BCD, I8254_BINARY
+
+  0 ID: INSN_CONFIG_8254_READ_STATUS
+  1 <-- Status byte returned here.
+    B7=Output
+    B6=NULL Count
+    B5-B0 Current mode.
+
+*/
+
+enum i8254_mode
+{
+	I8254_MODE0 = (0<<1),  /* Interrupt on terminal count */
+	I8254_MODE1 = (1<<1),  /* Hardware retriggerable one-shot */
+	I8254_MODE2 = (2<<1),  /* Rate generator */
+	I8254_MODE3 = (3<<1),  /* Square wave mode */
+	I8254_MODE4 = (4<<1),  /* Software triggered strobe */
+	I8254_MODE5 = (5<<1),  /* Hardware triggered strobe (retriggerable) */
+	I8254_BCD = 1, /* use binary-coded decimal instead of binary (pretty useless) */
+	I8254_BINARY = 0
+};
 #ifdef __cplusplus
 }
 #endif
