@@ -13,5 +13,10 @@
 #include <linux/pci.h>
 #endif
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,3,0)
+/* FIXME drivers are broken with __ISAPNP__ in 2.6. */
+#undef __ISAPNP__
+#endif
+
 #endif
 
