@@ -15,14 +15,6 @@
 #include_next <linux/pci.h>
 #define PCI_SUPPORT_VER2
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,0)
-/* we should get rid of this, as it has been dropped from 2.6 */
-#define pci_for_each_dev(x) \
-	for((x) = pci_find_device(PCI_ANY_ID,PCI_ANY_ID,NULL); \
-		(x); \
-		(x) = pci_find_device(PCI_ANY_ID,PCI_ANY_ID,(x)))
-#endif
-
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,3,0) /* XXX */
 
 struct pci_driver {
@@ -39,9 +31,6 @@ struct pci_driver {
 static inline int pci_module_init(struct pci_driver *drv) { return 0; }
 static inline void pci_unregister_driver(struct pci_driver *) { return; }
 
-#define pci_for_each_dev(x)	\
-	for((x)=pci_devices;(x);(x)=(x)->next)
-
 static inline int pci_enable_device(struct pci_dev *dev){return 0;}
 #define pci_disable_device(x)
 
@@ -153,15 +142,35 @@ static inline void pci_release_regions(struct pci_dev *dev)
 	}
 }
 
+#endif
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,3,15)
 static inline struct pci_dev* pci_find_subsys( unsigned int vendor, unsigned int device,
 	unsigned int ss_vendor, unsigned int ss_device,
 	const struct pci_dev *from)
 {
-	rt_printk( "pci_find_subsys() not supported in kernels older than 2.4!\n" );
-	return NULL;
-}
+	struct pci_dev *dev = (struct pci_dev *)from;	/* ignore const */
 
-#endif
+	while((dev=pci_find_device(vendor,device,dev))!=NULL){
+		if(ss_vendor!=PCI_ANY_ID){
+			u16 temp;
+			pci_read_config_word(dev,PCI_SUBSYSTEM_VENDOR_ID,&temp);
+			if(ss_vendor!=temp){
+				continue;
+			}
+		}
+		if(ss_device!=PCI_ANY_ID){
+			u16 temp;
+			pci_read_config_word(dev,PCI_SUBSYSTEM_ID,&temp);
+			if(ss_device!=temp){
+				continue;
+			}
+		}
+		break;	/* match */
+	}
+	return dev;
+}
+#endif	// <2.3.15
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,2,14)
 static inline void pci_set_master(struct pci_dev *dev)
@@ -173,6 +182,13 @@ static inline unsigned long pci_resource_len (struct pci_dev *dev, int n_base)
 { return 0; }
 #endif	// 2.2.18
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
+#define pci_get_device pci_find_device
+#define pci_get_subsys pci_find_subsys
+#define pci_dev_get(x)	(x)
+#define pci_dev_put(x)
+#endif
+
 #endif
 
 #endif /* _COMPAT_PCI_H */
